# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import mock

from openstack.tests.unit import test_proxy_base
from otcextensions.sdk.dws.v1 import _proxy
from otcextensions.sdk.dws.v1 import cluster as _cluster
from otcextensions.sdk.dws.v1 import flavor as _flavor
from otcextensions.sdk.dws.v1 import snapshot as _snapshot
from otcextensions.sdk.dws.v1 import tag as _tag

ENDPOINT_DWS = 'http://dws.example.com/v1.0'


class TestDwsProxy(test_proxy_base.TestProxyBase):
    def setUp(self):
        super(TestDwsProxy, self).setUp()
        self.proxy = _proxy.Proxy(self.session)
        self.session.get_endpoint = mock.Mock(return_value=ENDPOINT_DWS)

    def test_clusters(self):
        self.verify_list(
            self.proxy.clusters,
            _cluster.Cluster,
        )

    def test_get_cluster(self):
        self.verify_get(self.proxy.get_cluster, _cluster.Cluster)

    def test_find_cluster(self):
        self.verify_find(self.proxy.find_cluster, _cluster.Cluster, False)

    def test_create_cluster(self):
        self.verify_create(
            self.proxy.create_cluster,
            _cluster.Cluster,
            method_kwargs={'x': 1, 'y': 2, 'z': 3},
            expected_kwargs={'x': 1, 'y': 2, 'z': 3},
        )

    def test_delete_cluster(self):
        self._verify(
            'openstack.proxy.Proxy._delete',
            self.proxy.delete_cluster,
            method_args=['cluster-uuid', 0, True],
            expected_args=[_cluster.Cluster, 'cluster-uuid'],
            expected_kwargs={
                'keep_last_manual_snapshot': 0,
                'ignore_missing': True,
            },
        )

    def test_restart_cluster(self):
        self._verify(
            'otcextensions.sdk.dws.v1.cluster.Cluster.restart',
            self.proxy.restart_cluster,
            method_args=[_cluster.Cluster],
            expected_args=[self.proxy],
        )

    def test_scale_out_cluster(self):
        self._verify(
            'otcextensions.sdk.dws.v1.cluster.Cluster.scale_out',
            self.proxy.scale_out_cluster,
            method_args=[_cluster.Cluster, 2],
            expected_args=[self.proxy, 2],
        )

    def test_reset_password(self):
        self._verify(
            'otcextensions.sdk.dws.v1.cluster.Cluster.reset_password',
            self.proxy.reset_password,
            method_args=[_cluster.Cluster, 'TestNewPassword'],
            expected_args=[self.proxy, 'TestNewPassword'],
        )

    def test_snapshots(self):
        self.verify_list(
            self.proxy.snapshots,
            _snapshot.Snapshot,
        )

    def test_get_snapshot(self):
        self.verify_get(self.proxy.get_snapshot, _snapshot.Snapshot)

    def test_find_snapshot(self):
        self.verify_find(self.proxy.find_snapshot, _snapshot.Snapshot, False)

    def test_create_snapshot(self):
        self.verify_create(
            self.proxy.create_snapshot,
            _snapshot.Snapshot,
            method_kwargs={'x': 1, 'y': 2},
            expected_kwargs={'x': 1, 'y': 2},
        )

    def test_restore_snapshot(self):
        self.verify_create(
            self.proxy.restore_snapshot,
            _snapshot.Restore,
            method_args=['snapshot-uuid'],
            method_kwargs={'x': 1, 'y': 2, 'z': 3},
            expected_args=[],
            expected_kwargs={
                'snapshot_id': 'snapshot-uuid',
                'x': 1, 'y': 2, 'z': 3,
            },
        )

    def test_delete_snapshot(self):
        self._verify(
            'openstack.proxy.Proxy._delete',
            self.proxy.delete_snapshot,
            method_args=['snapshot-uuid', True],
            expected_args=[_snapshot.Snapshot, 'snapshot-uuid'],
            expected_kwargs={'ignore_missing': True},
        )

    def test_flavors(self):
        self.verify_list(
            self.proxy.flavors,
            _flavor.Flavor,
        )

    def test_cluster_tags(self):
        self.verify_list(
            self.proxy.cluster_tags,
            _tag.Tag,
            method_args=['test_cluster_id'],
            expected_kwargs={'cluster_id': 'test_cluster_id'},
            expected_args=[],
        )

    def test_create_cluster_tag(self):
        tag = {'key': 'key1', 'value': 'value1'}
        self.verify_create(
            self.proxy.create_cluster_tag,
            _tag.Tag,
            method_args=['test_cluster_id', tag],
            method_kwargs={},
            expected_kwargs={
                'cluster_id': 'test_cluster_id',
                'key': 'key1',
                'value': 'value1',
            },
            expected_args=[],
        )

    def test_delete_cluster_tag(self):
        self.verify_delete(
            self.proxy.delete_cluster_tag,
            _tag.Tag,
            method_args=['test_cluster_id', 'key1'],
            method_kwargs={},
            expected_kwargs={
                'cluster_id': 'test_cluster_id',
                'ignore_missing': True,
            },
            expected_args=['key1'],
        )

    # Tests for batch operations
    def test_batch_create_cluster_tags(self):
        self._verify(
            'otcextensions.sdk.dws.v1.tag.Tag.manage_tags_batch',
            self.proxy.batch_create_cluster_tags,
            method_args=[
                'test_cluster_id',
                [
                    {'key': 'key1', 'value': 'value1'},
                    {'key': 'key2', 'value': 'value2'},
                ],
            ],
            expected_args=[
                self.proxy,
                'test_cluster_id',
                [
                    {'key': 'key1', 'value': 'value1'},
                    {'key': 'key2', 'value': 'value2'},
                ],
                'create',
            ],
            expected_kwargs={},
        )

    def test_batch_delete_cluster_tags(self):
        self._verify(
            'otcextensions.sdk.dws.v1.tag.Tag.manage_tags_batch',
            self.proxy.batch_delete_cluster_tags,
            method_args=[
                'test_cluster_id',
                [{'key': 'key1'}, {'key': 'key2'}],
            ],
            expected_args=[
                self.proxy,
                'test_cluster_id',
                [{'key': 'key1'}, {'key': 'key2'}],
                'delete',
            ],
            expected_kwargs={},
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource
from openstack import exceptions


class ThrottlingPolicyRecords(resource.Resource):
    # API publication record ID.
    publish_id = resource.Body('publish_id', type=str)
    # Scope of the policy.
    # 1: the API
    # 2: a user
    # 3: an app
    scope = resource.Body('scope', type=int)
    # Request throttling policy ID.
    throttle_id = resource.Body('strategy_id')
    # Binding time.
    applied_at = resource.Body('apply_time', type=str)


class ThrottleBindingBatchFailure(resource.Resource):
    # ID of a request throttling policy binding record
    # that fails to be canceled.
    bind_id = resource.Body('bind_id', type=str)
    # Error code.
    error_code = resource.Body('error_code', type=int)
    # Error message.
    error_msg = resource.Body('error_msg')
    # ID of an API from which unbinding fails.
    api_id = resource.Body('api_id', type=str)
    # Name of the API from which unbinding fails.
    api_name = resource.Body('api_name', type=str)


class ThrottlingPolicyBind(resource.Resource):
    base_path = f'/apigw/instances/%(gateway_id)s/throttle-bindings'

    allow_list = True
    allow_create = True
    allow_delete = True

    resources_key = 'apis'

    _query_mapping = resource.QueryParameters(
        'limit', 'offset', 'throttle_id',
        'env_id', 'group_id', 'api_id', 'api_name'
    )

    # Properties
    gateway_id = resource.URI('gateway_id')
    # Request throttling policy ID.
    throttle_id = resource.Body('strategy_id')
    # API publication record ID.
    publish_ids = resource.Body('publish_ids', type=list)

    # Attributes
    # API publication record ID.
    publish_id = resource.Body('publish_id', type=str)
    # API authentication mode.
    auth_type = resource.Body('auth_type', type=str)
    # Name of the API group to which the API belongs.
    group_name = resource.Body('group_name', type=str)
    # ID of a request throttling policy binding record.
    throttle_apply_id = resource.Body('throttle_apply_id', type=str)
    # Binding time.
    applied_at = resource.Body('apply_time', type=str)
    # API description.
    remark = resource.Body('remark', type=str)
    # ID of the environment in which the API has been published.
    run_env_id = resource.Body('run_env_id', type=str)
    # API type.
    type = resource.Body('type', type=int)
    # Name of the request throttling policy bound to the API.
    throttle_name = resource.Body('throttle_name', type=str)
    # Access address of the API.
    req_uri = resource.Body('req_uri', type=str)
    # Name of the environment in which the API has been published.
    run_env_name = resource.Body('run_env_name', type=str)
    # ID of the API group to which the API belongs.
    group_id = resource.Body('group_id', type=str)
    # API name.
    name = resource.Body('name', type=str)
    # Request method.
    # Enumeration values:
    # GET
    # POST
    # DELETE
    # PUT
    # PATCH
    # HEAD
    # OPTIONS
    # ANY
    req_method = resource.Body('req_method', type=str)

    policies = resource.Body(
        'policies', type=list, list_type=ThrottlingPolicyRecords
    )
    failure = resource.Body(
        'failure', type=list, list_type=ThrottleBindingBatchFailure
    )

    def _action(self, session, body):
        """Preform actions given the message body.
        """
        url = self.base_path % {'gateway_id': body['gateway_id']}
        url += '?action=delete'
        response = session.put(url, json=body)
        exceptions.raise_from_response(response)
        self._translate_response(response)
        return self

    def unbind_policies(self, session, **attrs):
        """Unbind request throttling policies from APIs.
        """
        return self._action(session, attrs)

    def create(
            self,
            session,
            prepend_key=True,
            base_path=None,
            *,
            resource_request_key=None,
            resource_response_key=None,
            microversion=None,
            **params,
    ):
        """Create a remote resource based on this instance.

        :param session: The session to use for making this request.
        :type session: :class:`~keystoneauth1.adapter.Adapter`
        :param prepend_key: A boolean indicating whether the resource_key
            should be prepended in a resource creation request. Default to
            True.
        :param str base_path: Base part of the URI for creating resources, if
            different from :data:`~openstack.resource.Resource.base_path`.
        :param str resource_request_key: Overrides the usage of
            self.resource_key when prepending a key to the request body.
            Ignored if `prepend_key` is false.
        :param str resource_response_key: Overrides the usage of
            self.resource_key when processing response bodies.
            Ignored if `prepend_key` is false.
        :param str microversion: API version to override the negotiated one.
        :param dict params: Additional params to pass.
        :return: This :class:`Resource` instance.
        :raises: :exc:`~openstack.exceptions.MethodNotSupported` if
            :data:`Resource.allow_create` is not set to ``True``.
        """
        if not self.allow_create:
            raise exceptions.MethodNotSupported(self, 'create')

        session = self._get_session(session)
        if microversion is None:
            microversion = self._get_microversion(session)
        requires_id = (
            self.create_requires_id
            if self.create_requires_id is not None
            else self.create_method == 'PUT'
        )

        # Construct request arguments.
        request_kwargs = {
            "requires_id": requires_id,
            "prepend_key": prepend_key,
            "base_path": base_path,
        }
        if resource_request_key is not None:
            request_kwargs['resource_request_key'] = resource_request_key

        if self.create_exclude_id_from_body:
            self._body._dirty.discard("id")

        if self.create_method == 'PUT':
            request = self._prepare_request(**request_kwargs)
            response = session.put(
                request.url,
                json=request.body,
                headers=request.headers,
                microversion=microversion,
                params=params,
            )
        elif self.create_method == 'POST':
            request = self._prepare_request(**request_kwargs)
            response = session.post(
                request.url,
                json=request.body,
                headers=request.headers,
                microversion=microversion,
                params=params,
            )
        else:
            raise exceptions.ResourceFailure(
                "Invalid create method: %s" % self.create_method
            )

        has_body = (
            self.has_body
            if self.create_returns_body is None
            else self.create_returns_body
        )
        self.microversion = microversion

        response_kwargs = {
            "has_body": has_body,
        }
        if resource_response_key is not None:
            response_kwargs['resource_response_key'] = resource_response_key

        if has_body is None:
            has_body = self.has_body
        exceptions.raise_from_response(response)
        policies = []
        if has_body and response.status_code == 201:
            body = response.json()
            policies = [
                ThrottlingPolicyRecords.existing(**data)
                for data in body['throttle_applys']
            ]
        self.policies = policies
        # direct comparision to False since we need to rule out None
        if self.has_body and self.create_returns_body is False:
            # fetch the body if it's required but not returned by create
            fetch_kwargs = {}
            if resource_response_key is not None:
                fetch_kwargs = {'resource_response_key': resource_response_key}
            return self.fetch(session, **fetch_kwargs)
        return self


class NotBoundApi(resource.Resource):
    base_path = (f'/apigw/instances/%(gateway_id)s/'
                 f'throttle-bindings/unbinded-apis')

    allow_list = True

    resources_key = 'apis'

    _query_mapping = resource.QueryParameters(
        'limit', 'offset', 'throttle_id',
        'env_id', 'api_id', 'api_name', 'group_id'
    )

    # Properties
    gateway_id = resource.URI('gateway_id')

    # Attributes
    # API authentication mode.
    auth_type = resource.Body('auth_type', type=str)
    # Name of the environment in which the API has been published.
    run_env_name = resource.Body('run_env_name', type=str)
    # ID of a request throttling policy binding record.
    throttle_apply_id = resource.Body('throttle_apply_id', type=str)
    # Binding time.
    applied_at = resource.Body('apply_time', type=str)
    # Name of the API group to which the API belongs.
    group_name = resource.Body('group_name', type=str)
    # Publication record ID.
    publish_id = resource.Body('publish_id', type=str)
    # ID of the API group to which the API belongs.
    group_id = resource.Body('group_id', type=str)
    # API name.
    name = resource.Body('name', type=str)
    # API description.
    remark = resource.Body('remark', type=str)
    # ID of the environment in which the API has been published.
    run_env_id = resource.Body('run_env_id', type=str)
    # API request address.
    req_uri = resource.Body('req_uri', type=str)
    # API type.
    type = resource.Body('type', type=int)
    # Name of the request throttling policy bound to the API.
    throttle_name = resource.Body('throttle_name', type=str)
    # Request method.
    # Enumeration values:
    # GET
    # POST
    # DELETE
    # PUT
    # PATCH
    # HEAD
    # OPTIONS
    # ANY
    req_method = resource.Body('req_method', type=str)


class BoundThrottles(resource.Resource):
    base_path = (f'/apigw/instances/%(gateway_id)s/'
                 f'throttle-bindings/binded-throttles')

    allow_list = True

    resources_key = 'throttles'

    _query_mapping = resource.QueryParameters(
        'limit', 'offset', 'api_id',
        'throttle_id', 'throttle_name', 'env_id'
    )

    # Properties
    gateway_id = resource.URI('gateway_id')

    # Attributes
    # Maximum number of times the API can be accessed by an
    # app within the same period.
    app_call_limits = resource.Body('app_call_limits', type=int)
    # Request throttling policy name.
    name = resource.Body('name', type=str)
    # Time unit for limiting the number of API calls.
    # Enumeration values:
    # SECOND
    # MINUTE
    # HOUR
    # DAY
    time_unit = resource.Body('time_unit', type=str)
    # Description of the request throttling policy,
    # which can contain a maximum of 255 characters.
    remark = resource.Body('remark', type=str)
    # Maximum number of times an API can be accessed within a specified period.
    api_call_limits = resource.Body('api_call_limits', type=int)
    # Type of the request throttling policy.
    # 1: API-based, limiting the maximum number of times a single
    # API bound to the policy can be called within the specified period.
    # 2: API-shared, limiting the maximum number of times all
    # APIs bound to the policy can be called within the specified period.
    type = resource.Body('type', type=int)
    # Indicates whether to enable dynamic request throttling.
    enable_adaptive_control = resource.Body(
        'enable_adaptive_control', type=str
    )
    # Maximum number of times the API can be accessed
    # by a user within the same period.
    user_call_limits = resource.Body('user_call_limits', type=int)
    # Period of time for limiting the number of API calls.
    time_interval = resource.Body('time_interval', type=int)
    # Maximum number of times the API can be accessed
    # by an IP address within the same period.
    ip_call_limits = resource.Body('ip_call_limits', type=int)
    # Number of APIs to which the request throttling policy has been bound.
    bind_num = resource.Body('bind_num', type=int)
    # Indicates whether an excluded request throttling
    # configuration has been created.
    # 1: yes
    # 2: no
    is_inclu_special_throttle = resource.Body(
        'is_inclu_special_throttle', type=int
    )
    # Creation time.
    created_at = resource.Body('create_time', type=str)
    # Environment in which the request throttling policy takes effect.
    env_name = resource.Body('env_name', type=str)
    # Policy binding record ID.
    bind_id = resource.Body('bind_id', type=str)
    # Time when the policy is bound to the API.
    bound_at = resource.Body('bind_time', type=str)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import exceptions
from openstack import resource
from openstack import utils


class Peering(resource.Resource):
    resources_key = 'peerings'
    resource_key = 'peering'
    base_path = '/v2.0/vpc/peerings'

    # capabilities
    allow_create = True
    allow_fetch = True
    allow_commit = True
    allow_delete = True
    allow_list = True

    _query_mapping = resource.QueryParameters(
        'id', 'marker', 'limit', 'name', 'router_id',
        'status', 'project_id', project_id='tenant_id',
        router_id='vpc_id'
    )

    # Properties
    #: Specifies information about the local VPC.
    #: *Type: dict*
    peer_vpc_info = resource.Body('accept_vpc_info', type=dict)
    #: Specifies information about the local VPC.
    #: *Type: dict*
    local_vpc_info = resource.Body('request_vpc_info', type=dict)
    #: Specifies the status.
    status = resource.Body('status')
    #: Provides supplementary information about the VPC peering connection.
    description = resource.Body('description')
    #: Specifies the time (UTC) when the VPC peering connection is created.
    #:  Format is *yyyy-mm-dd hh:mm:ss*.
    created_at = resource.Body('created_at')
    #: Specifies the time (UTC) when the VPC peering connection is updated.
    #:  Format is *yyyy-mm-dd hh:mm:ss*.
    updated_at = resource.Body('updated_at')

    def _set_peering(self, session, set_status):
        """Accept/Reject Peering Request"""
        url = utils.urljoin(self.base_path, self.id, set_status)
        response = session.put(url)
        exceptions.raise_from_response(response)
        self._translate_response(response)
        return self



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import copy

from keystoneauth1 import adapter

import mock

from openstack.tests.unit import base

from otcextensions.sdk.rds.v1 import configuration

OS_HEADERS = {
    'Content-Type': 'application/json',
}

ENDPOINT = 'http://some_endpoint'

PROJECT_ID = '23'
IDENTIFIER = 'IDENTIFIER'
EXAMPLE_GROUP = {
    'id': IDENTIFIER,
    'name': 'default-SQLServer-2014',
    'datastore_version_id': '4f71c5b5-8939-424e-8825-8e3816e4303d',
    'datastore_version_name': '2014',
    'datastore_name': 'sqlserver',
    'description': 'Default parameter group for sqlserver 2014',
    'instance_count': 0,
    'created': '2017-05-05T04:40:51',
    'updated': '2017-05-05T04:40:51',
    'values': {
        'xp_cmdshell': '0'
    },
    'parameters': [{
        'name': 'auto_increment_increment',
        'value': '1',
        'needRestart': '0',
        'readonly': '1',
        'valueRange': '1-65535',
        'datatype': 'integer',
        'description': 'auto_increment_increment and auto_increment_offset.'
    }, {
        'name': 'autocommit',
        'value': 'ON',
        'needRestart': '0',
        'readonly': '1',
        'valueRange': 'ON|OFF',
        'datatype': 'boolean',
        'description': 'The autocommit mode. If set to ON, all changes'
    }
    ],
}


class TestConfigurationGroup(base.TestCase):

    # TODO(agoncharov)
    # - test all fields
    # - test negative server responses

    def setUp(self):
        super(TestConfigurationGroup, self).setUp()
        self.sess = mock.Mock(spec=adapter.Adapter)
        self.sess.get = mock.Mock()
        self.sess.post = mock.Mock()
        self.sess.delete = mock.Mock()
        self.sess.put = mock.Mock()
        self.sess.patch = mock.Mock()
        self.sess.get_project_id = mock.Mock(return_value=PROJECT_ID)
        self.sot = configuration.ConfigurationGroup(**EXAMPLE_GROUP)

    def test_basic(self):
        sot = configuration.ConfigurationGroup()
        self.assertEqual('configuration', sot.resource_key)
        self.assertEqual('configurations', sot.resources_key)
        self.assertEqual('/%(project_id)s/configurations', sot.base_path)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_get)
        self.assertTrue(sot.allow_update)
        self.assertTrue(sot.allow_delete)

    def test_make_it(self):
        # TODO(agoncharov) check all parameters
        sot = configuration.ConfigurationGroup(**EXAMPLE_GROUP)
        self.assertEqual(IDENTIFIER, sot.id)
        self.assertEqual(EXAMPLE_GROUP['name'], sot.name)
        self.assertEqual(EXAMPLE_GROUP['created'], sot.created)

    def test_list(self):

        mock_response = mock.Mock()
        mock_response.status_code = 200
        mock_response.json.return_value = {'configurations': [EXAMPLE_GROUP]}

        self.sess.get.return_value = mock_response

        result = list(self.sot.list(
            self.sess,
            project_id=PROJECT_ID,
            headers=OS_HEADERS
        ))

        self.sess.get.assert_called_once_with(
            '/%s/configurations' % (PROJECT_ID),
            headers=OS_HEADERS,
            params={}
        )

        self.assertEqual([configuration.ConfigurationGroup(**EXAMPLE_GROUP)],
                         result)

    def test_get(self):

        mock_response = mock.Mock()
        mock_response.status_code = 200
        mock_response.json.return_value = {
            'configuration': copy.deepcopy(EXAMPLE_GROUP)
        }
        mock_response.headers = {}

        self.sess.get.return_value = mock_response

        sot = configuration.ConfigurationGroup.new(
            project_id=PROJECT_ID,
            id=IDENTIFIER,
            # **EXAMPLE_GROUP
        )

        result = sot.get(self.sess, headers=OS_HEADERS)

        self.sess.get.assert_called_once_with(
            '%s/configurations/%s' % (PROJECT_ID, IDENTIFIER),
            headers=OS_HEADERS,
        )

        self.assertEqual(
            configuration.ConfigurationGroup(
                project_id=PROJECT_ID,
                **EXAMPLE_GROUP
            ),
            result)

    def test_create(self):

        mock_response = mock.Mock()
        mock_response.status_code = 200
        mock_response.json.return_value = {
            'configuration': copy.deepcopy(EXAMPLE_GROUP)
        }
        mock_response.headers = {}

        self.sess.post.return_value = mock_response

        sot = configuration.ConfigurationGroup.new(
            project_id=PROJECT_ID,
            **EXAMPLE_GROUP)

        result = sot.create(self.sess, headers=OS_HEADERS)

        self.sess.post.assert_called_once_with(
            '/%s/configurations' % (PROJECT_ID),
            headers=OS_HEADERS,
            json={'configuration': EXAMPLE_GROUP}
        )

        self.assertEqual(
            configuration.ConfigurationGroup(
                project_id=PROJECT_ID,
                **EXAMPLE_GROUP
            ),
            result)

        # TODO(agoncharov)
        # {"badRequest":{"code":400,"message":
        # "The server could not comply with the request since it is malformed
        # or incorrect."},"errCode":"RDS.0001","externalMessage":
        # "Parameter error!"}

    def test_delete(self):

        mock_response = mock.Mock()
        mock_response.status_code = 200
        mock_response.json.return_value = {}
        mock_response.headers = {}

        self.sess.delete.return_value = mock_response

        sot = configuration.ConfigurationGroup(
            project_id=PROJECT_ID,
            **EXAMPLE_GROUP
        )

        sot.delete(self.sess, headers=OS_HEADERS)

        url = '%(project_id)s/configurations/%(id)s' % \
            {
                'project_id': PROJECT_ID,
                'id': sot.id
            }

        # utils.urljoin strips leading '/', but it is not a problem
        self.sess.delete.assert_called_once_with(
            url,
            headers=OS_HEADERS
        )

    def _verify2(self, mock_method, test_method,
                 method_args=None, method_kwargs={}, method_result=None,
                 expected_args=None, expected_kwargs=None,
                 expected_result=None):
        """Internal invoke helper

        """

        mock_method.reset_mock()
        mock_method.return_value = expected_result

        test_method(*method_args, **method_kwargs)

        mock_method.assert_called_once_with(
            *expected_args,
            **expected_kwargs
        )

    def test_get_associated_instances(self):
        mock_response = mock.Mock()
        mock_response.status_code = 200
        mock_response.json.return_value = []
        mock_response.headers = {}

        self.sess.get.return_value = mock_response

        sot = configuration.ConfigurationGroup(
            project_id=PROJECT_ID,
            **EXAMPLE_GROUP
        )

        url = '%(project_id)s/configurations/%(id)s/instances' % \
            {
                'project_id': PROJECT_ID,
                'id': sot.id
            }

        # Invoke without endpoint_override
        self._verify2(
            expected_result=mock_response,
            mock_method=self.sess.get,
            test_method=sot.get_associated_instances,
            method_args=[self.sess],
            method_kwargs={},
            expected_args=[url],
            expected_kwargs={
                'headers': OS_HEADERS,
            }
        )

        # Invoke with endpoint_override as argument
        self._verify2(
            expected_result=mock_response,
            mock_method=self.sess.get,
            test_method=sot.get_associated_instances,
            method_args=[self.sess],
            method_kwargs={'endpoint_override': ENDPOINT},
            expected_args=[url],
            expected_kwargs={
                'headers': OS_HEADERS,
                'endpoint_override': ENDPOINT
            }
        )

        # Invoke with endpoint_override as attribute
        sot.endpoint_override = ENDPOINT
        self._verify2(
            expected_result=mock_response,
            mock_method=self.sess.get,
            test_method=sot.get_associated_instances,
            method_args=[self.sess],
            method_kwargs={},
            expected_args=[url],
            expected_kwargs={
                'headers': OS_HEADERS,
                'endpoint_override': ENDPOINT
            }
        )

    # def test_add_custom_parameter(self):
    #     mock_response = mock.Mock()
    #     mock_response.status_code = 200
    #     mock_response.json.return_value = {
    #         'errCode': 'RDS.0041',
    #         'externalMessage': 'Operation accepted success.'
    #     }
    #     mock_response.headers = {}
    #
    #     config = {
    #         'values': {
    #             'a': 'x',
    #             'b': 'y'
    #         }
    #     }
    #
    #     req = {
    #         'configuration': dict(**config)
    #     }
    #
    #     sot = configuration.ConfigurationGroup(
    #         project_id=PROJECT_ID,
    #         **EXAMPLE_GROUP
    #     )
    #
    #     url = '%(project_id)s/configurations/%(id)s' % \
    #         {
    #             'project_id': PROJECT_ID,
    #             'id': sot.id
    #         }
    #
    #     # Invoke without endpoint_override
    #     self._verify2(
    #         expected_result=mock_response,
    #         mock_method=self.sess.patch,
    #         test_method=sot.add_custom_parameter,
    #         method_args=[self.sess],
    #         method_kwargs=config,
    #         expected_args=[url],
    #         expected_kwargs={
    #             'json': req,
    #             # 'headers': OS_HEADERS,
    #         }
    #     )
    #
    #     # Invoke with endpoint_override as argument
    #     self._verify2(
    #         expected_result=mock_response,
    #         mock_method=self.sess.patch,
    #         test_method=sot.add_custom_parameter,
    #         method_args=[self.sess],
    #         method_kwargs=dict(endpoint_override=ENDPOINT, **config),
    #         expected_args=[url],
    #         expected_kwargs={
    #             'json': req,
    #             # 'headers': OS_HEADERS,
    #             'endpoint_override': ENDPOINT
    #         }
    #     )
    #
    #     # Invoke with endpoint_override as attribute
    #     sot.endpoint_override = ENDPOINT
    #     self._verify2(
    #         expected_result=mock_response,
    #         mock_method=self.sess.patch,
    #         test_method=sot.add_custom_parameter,
    #         method_args=[self.sess],
    #         method_kwargs=config,
    #         expected_args=[url],
    #         expected_kwargs={
    #             'json': req,
    #             # 'headers': OS_HEADERS,
    #             'endpoint_override': ENDPOINT
    #         }
    #     )

    # def test_update(self):
    #     mock_response = mock.Mock()
    #     mock_response.status_code = 200
    #     mock_response.json.return_value = {
    #         'errCode': 'RDS.0041',
    #         'externalMessage': 'Operation accepted success.'
    #     }
    #     mock_response.headers = {}
    #
    #     config = {
    #         'name': 'test',
    #         'description': 'descr',
    #         'values': {
    #             'a': 'x',
    #             'b': 'y'
    #         }
    #     }
    #
    #     req = {
    #         'configuration': dict(**config)
    #     }
    #
    #     sot_tmpl = configuration.ConfigurationGroup.existing(
    #         project_id=PROJECT_ID,
    #         **EXAMPLE_GROUP
    #     )
    #
    #     sot = copy.deepcopy(sot_tmpl)
    #     sot._update(**config)
    #
    #     url = '%(project_id)s/configurations/%(id)s' % \
    #         {
    #             'project_id': PROJECT_ID,
    #             'id': sot.id
    #         }
    #
    #     # Invoke without endpoint_override
    #     self._verify2(
    #         expected_result=mock_response,
    #         mock_method=self.sess.put,
    #         test_method=sot.update,
    #         method_args=[self.sess],
    #         # method_kwargs=None,
    #         expected_args=[url],
    #         expected_kwargs={
    #             'json': req,
    #         }
    #     )
    #
    #     sot = copy.deepcopy(sot_tmpl)
    #     sot._update(**config)
    #
    #     # Invoke with endpoint_override as argument
    #     self._verify2(
    #         expected_result=mock_response,
    #         mock_method=self.sess.put,
    #         test_method=sot.update,
    #         method_args=[self.sess],
    #         method_kwargs=dict(endpoint_override=ENDPOINT),
    #         expected_args=[url],
    #         expected_kwargs={
    #             'json': req,
    #             'endpoint_override': ENDPOINT
    #         }
    #     )
    #
    #     sot = copy.deepcopy(sot_tmpl)
    #     sot._update(**config)



#!/usr/bin/env python3
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
'''
Update MRS cluster
'''
import openstack

openstack.enable_logging(True)
conn = openstack.connect(cloud='otc')


attrs = {
    'parameters': {
        'scale_type': 'scale_out',
        'node_id': 'node_orderadd',
        'instances': "1",
        'node_group': 'task_node_default_group'
    }
}

cluster = 'clstr1'
cluster = conn.mrs.find_cluster(name_or_id=cluster)
cluster = conn.mrs.update_cluster(cluster=cluster, **attrs)
print(cluster)



#!/usr/bin/env python3
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
'''
Add CloudEye metric data
'''
import openstack

openstack.enable_logging(True)
conn = openstack.connect(cloud='otc')

# not working due to lag of compatible API: list instead of proper JSON
attrs = [
    {
        "metric": {
            "namespace": "MINE.APP",
            "dimensions": [
                {
                    "name": "instance_id",
                    "value": "33328f02-3814-422e-b688-bfdba93d4050"
                }
            ],
            "metric_name": "cpu_util"
        },
        "ttl": 172800,
        "collect_time": 1598266684000,
        "type": "int",
        "value": 60,
        "unit": "%"
    },
    {
        "metric": {
            "namespace": "MINE.APP",
            "dimensions": [
                {
                    "name": "instance_id",
                    "value": "33328f02-3814-422e-b688-bfdba93d4050"
                }
            ],
            "metric_name": "cpu_util"
        },
        "ttl": 172800,
        "collect_time": 1598266685000,
        "type": "int",
        "value": 70,
        "unit": "%"
    }
]


data = conn.ces.create_metric_data(**attrs)
print(data)



#!/usr/bin/env python3
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""
Restore Snapshot to a DWS cluster to a newly created one.
"""
import openstack

openstack.enable_logging(True)
conn = openstack.connect(cloud='otc')

attrs = {
    'name': 'dws-1',
    'availability_zone': 'eu-de-01',
    'port': 8000,
    'vpc_id': 'router-uuid',
    'subnet_id': 'network-uuid',
    'security_group_id': 'security-group-uuid',
    'public_ip': {
        'public_bind_type': 'auto_assign',
        'eip_id': ''
    }
}

snapshot_id = 'snapshot-uuid'

response = conn.dws.restore_snapshot(snapshot_id, **attrs)
print(response)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource
from openstack import exceptions


class Quota(resource.Resource):
    base_path = f'/apigw/instances/%(gateway_id)s/apps/%(app_id)s/bound-quota'

    gateway_id = resource.URI('gateway_id')
    app_id = resource.URI('app_id')

    allow_fetch = True

    app_quota_id = resource.Body('app_quota_id')
    name = resource.Body('name')
    call_limits = resource.Body('call_limits', type=int)
    time_unit = resource.Body('time_unit')
    time_interval = resource.Body('time_interval')
    remark = resource.Body('remark')
    reset_time = resource.Body('reset_time')
    create_time = resource.Body('create_time')
    bound_app_num = resource.Body('bound_app_num')


class AccessControl(resource.Resource):
    base_path = f'/apigw/instances/%(gateway_id)s/apps/%(app_url_id)s/app-acl'
    gateway_id = resource.URI('gateway_id')
    app_url_id = resource.URI('app_url_id')

    app_id = resource.Body('app_id')
    app_acl_type = resource.Body('app_acl_type')
    app_acl_values = resource.Body('app_acl_values', type=list)

    allow_fetch = True
    allow_delete = True
    allow_update = True

    def _configure(self, session, gateway, app, **attrs):
        url = f'/apigw/instances/{gateway.id}/apps/{app.id}/app-acl'
        response = session.put(url, json=attrs)
        exceptions.raise_from_response(response)
        self._translate_response(response)
        return self

    def _delete(self, session, gateway, app, **attrs):
        url = f'/apigw/instances/{gateway.id}/apps/{app.id}/app-acl'
        response = session.delete(url, json=attrs)
        exceptions.raise_from_response(response)
        return None


class App(resource.Resource):
    base_path = f'/apigw/instances/%(gateway_id)s/apps'
    resources_key = 'apps'

    allow_fetch = True
    allow_list = True
    allow_create = True
    allow_commit = True
    allow_delete = True

    _query_mapping = resource.QueryParameters(
        'limit', 'offset', 'id', 'creator',
        'name', 'status', 'app_key', 'precise_search'
    )

    gateway_id = resource.URI('gateway_id')

    name = resource.Body('name')
    remark = resource.Body('remark')
    app_key = resource.Body('app_key')
    app_secret = resource.Body('app_secret')

    id = resource.Body('id')
    creator = resource.Body('creator')
    update_time = resource.Body('update_time')
    register_time = resource.Body('register_time')
    status = resource.Body('status', type=int)
    app_type = resource.Body('app_type')
    roma_app_type = resource.Body('roma_app_type')
    bind_num = resource.Body('bind_num', type=int)

    def _verify_app(self, session, gateway):
        url = f'/apigw/instances/{gateway.id}/apps/validation/{self.id}'
        response = session.get(url)
        exceptions.raise_from_response(response)
        self._translate_response(response)
        return self

    def _reset_secret(self, session, gateway, **attrs):
        url = f'/apigw/instances/{gateway.id}/apps/secret/{self.id}'
        response = session.put(url, json=attrs)
        exceptions.raise_from_response(response)
        self._translate_response(response)
        return self



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import os
from openstack import _log
from otcextensions.tests.functional import base

_logger = _log.setup_logging('openstack')


class TestClusterTags(base.BaseFunctionalTest):
    """Functional tests for DWS cluster tags."""

    def setUp(self):
        """Prepare resources for testing."""
        super(TestClusterTags, self).setUp()
        self.cluster_id = os.getenv("OS_DWS_CLUSTER_ID")
        if not self.cluster_id:
            TestClusterTags.skipTest(
                self, 'OS_DWS_CLUSTER_ID necessary for this test'
            )
        self.client = self.conn.dws
        self.created_tags = []

    def tearDown(self):
        """Clean up resources after testing."""
        for tag_key in self.created_tags:
            self.client.delete_cluster_tag(self.cluster_id, tag_key)
        self.created_tags.clear()
        super(TestClusterTags, self).tearDown()

    def test_cluster_tags(self):
        """Test creating, listing, and deleting a tag."""
        tag_key = "test_key"
        tag_value = "test_value"
        self.client.create_cluster_tag(
            self.cluster_id, {"key": tag_key, "value": tag_value})
        self.created_tags.append(tag_key)

        tags = list(self.client.cluster_tags(self.cluster_id))
        self.assertIn(tag_key, [tag.key for tag in tags])
        self.assertGreaterEqual(len(tags), 1)

    def test_create_cluster_tag(self):
        """Test creating a single tag for a cluster."""
        tag_key = "test_key"
        tag_value = "test_value"
        created_tag = self.client.create_cluster_tag(
            self.cluster_id, {"key": tag_key, "value": tag_value})
        self.assertIsNotNone(created_tag)
        self.assertEqual(created_tag.key, tag_key)
        self.assertEqual(created_tag.value, tag_value)
        self.created_tags.append(tag_key)

    def test_delete_cluster_tag(self):
        """Test deleting a single tag from a cluster."""
        tag_key = "delete_test_key"
        tag_value = "delete_test_value"
        self.client.create_cluster_tag(
            self.cluster_id, {"key": tag_key, "value": tag_value})
        self.client.delete_cluster_tag(self.cluster_id, tag_key)
        tags = list(self.client.cluster_tags(self.cluster_id))
        self.assertFalse(any(tag.key == tag_key for tag in tags))

    def test_batch_create_cluster_tags(self):
        """Test batch creation of multiple tags for a cluster."""
        tags_to_create = [
            {"key": "test_key1", "value": "test_value1"},
            {"key": "test_key2", "value": "test_value2"}
        ]
        created_tags = self.client.batch_create_cluster_tags(
            self.cluster_id, tags_to_create)
        self.assertIsInstance(created_tags, list)
        for tag in created_tags:
            self.assertIn(tag.key, [t["key"] for t in tags_to_create])
            self.created_tags.append(tag.key)

    def test_batch_delete_cluster_tags(self):
        """Test batch deletion of multiple tags from a cluster."""
        tags_to_create = [
            {"key": "batch_delete_key1", "value": "value1"},
            {"key": "batch_delete_key2", "value": "value2"}
        ]
        self.client.batch_create_cluster_tags(self.cluster_id, tags_to_create)
        tags_to_delete = [
            {"key": "batch_delete_key1"},
            {"key": "batch_delete_key2"}
        ]
        deleted_tags = self.client.batch_delete_cluster_tags(
            self.cluster_id, tags_to_delete)
        self.assertIsInstance(deleted_tags, list)
        for tag in deleted_tags:
            self.assertIn(tag.key, [t["key"] for t in tags_to_delete])



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import resource

from otcextensions.sdk.elb.v2 import _base


class Certificate(_base.Resource):
    resources_key = 'certificates'
    base_path = ('/lbaas/certificates')

    # capabilities
    allow_create = True
    allow_fetch = True
    allow_commit = True
    allow_delete = True
    allow_list = True

    _query_mapping = resource.QueryParameters(
        'id', 'name', 'description',
        'type', 'domain', 'content',
        'private_key', 'marker', 'limit',
    )

    # Properties
    #: Name
    name = resource.Body('name')
    #: Id
    id = resource.Body('id')
    #: Description
    description = resource.Body('description')
    #: Certificate type.
    type = resource.Body('type')
    #: Domain name associated with the server certificate.
    domain = resource.Body('domain')
    #: Private key of the server certificate. *Type: string*
    private_key = resource.Body('private_key')
    #: Public key of the server certificate or CA certificate. *Type: string*
    content = resource.Body('certificate')
    #: Administrative status of the certificate.
    admin_state_up = resource.Body('admin_state_up')
    #: Creation time
    create_time = resource.Body('create_time')
    #: Specifies the project ID.
    project_id = resource.Body('tenant_id')
    #: Time when the certificate expires.
    expire_time = resource.Body('expire_time')
    #: Time when the certificate was updated.
    update_time = resource.Body('update_time')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import uuid

import mock
from keystoneauth1 import adapter

from openstack.tests.unit import base
from otcextensions.sdk.dws.v1 import cluster

EXAMPLE = {
    'action_progress': {},
    'availability_zone': 'eu-de-01',
    'created': '2023-01-16T02:11:19',
    # "elb": None,
    'endpoints': [
        {
            'connect_info': 'test-dws-6433a577.dws.otc-tsi.de',
            'jdbc_url': 'jdbc:postgresql://test.example.com:8000/gaussdb',
        }
    ],
    'enterprise_project_id': uuid.uuid4().hex,
    'guest_agent_version': '8.1.1.202',
    'id': uuid.uuid4().hex,
    'logical_cluster_initialed': False,
    'logical_cluster_mode': False,
    'maintain_window': {
        'day': 'Thu',
        'end_time': '02:00',
        'start_time': '22:00',
    },
    'name': 'test-dws-6433a577',
    'node_type': 'dws.m3.xlarge',
    'node_type_id': uuid.uuid4().hex,
    'nodes': [
        {'id': uuid.uuid4().hex, 'status': '200'},
        {'id': uuid.uuid4().hex, 'status': '200'},
        {'id': uuid.uuid4().hex, 'status': '200'},
    ],
    'number_of_free_node': 0,
    'number_of_node': 6,
    'parameter_group': {
        'id': uuid.uuid4().hex,
        'name': 'parameterGroupFor_bd5cdd60-0c94-4f38-9a23-49cb2c5c425a',
        'status': 'In-Sync',
    },
    'plugins': [
        {
            'aarchType': 'x86_64',
            'autoInstall': 'install|upgrade',
            'datastoreType': 'dws',
            'datastoreVersion': '8.1.1.202',
            'id': uuid.uuid4().hex,
            'packageName': '8.1.1.202-dmsAgent-x86_64-20220211161451.tar.gz',
            'status': '0',
            'type': 'dmsAgent',
            'updateTime': 1666792707000,
            'version': '8.1.1.202',
        },
        {
            'aarchType': 'all',
            'autoInstall': 'install|upgrade',
            'datastoreType': 'dws',
            'datastoreVersion': '8.1.1.202',
            'id': uuid.uuid4().hex,
            'packageName': '8.1.1.202-inspect-ed1eb6e-20220207110934.tar.gz',
            'status': '0',
            'type': 'inspect',
            'updateTime': 1666792707000,
            'version': '8.1.1.202',
        },
    ],
    'port': 8000,
    'private_ip': ['192.168.1.192', '192.168.1.15', '192.168.1.73'],
    'public_endpoints': [
        {
            'jdbc_url': 'jdbc:postgresql://test.example.com:8000/gaussdb',
            'public_connect_info': 'test.example.com',
        }
    ],
    'public_ip': {
        'eip_address': '80.158.89.54',
        'eip_id': uuid.uuid4().hex,
        'public_bind_type': 'auto_assign',
    },
    'recent_event': 9,
    'security_group_id': uuid.uuid4().hex,
    'spec_version': 'v1.0',
    'status': 'AVAILABLE',
    'sub_status': 'NORMAL',
    'subnet_id': uuid.uuid4().hex,
    'tags': [
        {'key': 'key1', 'value': 'value1'},
        {'key': 'key2', 'value': 'value2'},
    ],
    'task_status': 'SNAPSHOTTING',
    'updated': '2023-01-16T02:11:19',
    'use_logical_cluster': False,
    'user_name': 'dbadmin',
    'version': '8.1.1.202',
    'vpc_id': uuid.uuid4().hex,
    'failed_reasons': {},
}


class TestCluster(base.TestCase):
    def setUp(self):
        super(TestCluster, self).setUp()
        self.sess = mock.Mock(spec=adapter.Adapter)

    def test_basic(self):
        sot = cluster.Cluster()

        self.assertEqual('/clusters', sot.base_path)
        self.assertEqual('clusters', sot.resources_key)
        self.assertEqual('cluster', sot.resource_key)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_commit)

    def test_make_it(self):
        sot = cluster.Cluster(**EXAMPLE)
        updated_sot_attrs = (
            'vpc_id',
            'subnet_id',
            'node_type',
            'node_type_id',
            'public_ip',
            'updated',
            'created',
            'failed_reasons',
            'use_logical_cluster',
            'logical_cluster_initialed',
            'logical_cluster_mode',
            'logical_cluster_mode',
            'maintain_window',
            'number_of_free_node',
            'number_of_node',
            'recent_event',
        )

        self.assertEqual(EXAMPLE['vpc_id'], sot.router_id)
        self.assertEqual(EXAMPLE['subnet_id'], sot.network_id)
        self.assertEqual(EXAMPLE['node_type'], sot.flavor)
        self.assertEqual(EXAMPLE['node_type_id'], sot.flavor_id)

        self.assertEqual(EXAMPLE['public_ip'], sot.floating_ip)
        self.assertEqual(EXAMPLE['created'], sot.created_at)
        self.assertEqual(EXAMPLE['updated'], sot.updated_at)
        self.assertEqual(EXAMPLE['failed_reasons'], sot.error)
        self.assertEqual(
            EXAMPLE['use_logical_cluster'], sot.is_logical_cluster_enabled
        )
        self.assertEqual(
            EXAMPLE['logical_cluster_initialed'],
            sot.is_logical_cluster_initialed,
        )
        self.assertEqual(
            EXAMPLE['logical_cluster_mode'], sot.is_logical_cluster_mode
        )
        self.assertEqual(EXAMPLE['maintain_window'], sot.maintenance_window)
        self.assertEqual(EXAMPLE['number_of_free_node'], sot.num_free_nodes)
        self.assertEqual(EXAMPLE['number_of_node'], sot.num_nodes)
        self.assertEqual(EXAMPLE['recent_event'], sot.num_recent_events)

        for key, value in EXAMPLE.items():
            if key not in updated_sot_attrs:
                self.assertEqual(getattr(sot, key), value)

    def test_action(self):
        sot = cluster.Cluster.existing(id=EXAMPLE['id'])
        action = 'restart'
        json_body = {'restart': {}}
        response = mock.Mock()
        response.status_code = 200
        response.headers = {}
        self.sess.post.return_value = response

        rt = sot._action(self.sess, action, json_body)
        self.sess.post.assert_called_with(
            'clusters/%s/restart' % sot.id, json=json_body
        )

        self.assertIsNone(rt)

    def test_restart(self):
        sot = cluster.Cluster.existing(id=EXAMPLE['id'])
        sot._action = mock.Mock()

        rt = sot.restart(self.sess)
        sot._action.assert_called_with(self.sess, 'restart', {'restart': {}})
        self.assertIsNone(rt)

    def test_scale_out(self):
        sot = cluster.Cluster.existing(id=EXAMPLE['id'])
        sot._action = mock.Mock()
        node_count = 3

        rt = sot.scale_out(self.sess, node_count)
        sot._action.assert_called_with(
            self.sess, 'resize', {'scale_out': {'count': node_count}}
        )
        self.assertIsNone(rt)

    def test_reset_password(self):
        sot = cluster.Cluster.existing(id=EXAMPLE['id'])
        sot._action = mock.Mock()
        new_password = 'TestNewPassword'

        rt = sot.reset_password(self.sess, new_password)
        sot._action.assert_called_with(
            self.sess, 'reset-password', {'new_password': new_password}
        )
        self.assertIsNone(rt)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import proxy
from otcextensions.sdk.volume_backup.v2 import backup_policy as _backup_policy
from otcextensions.sdk.volume_backup.v2 import backup_task as _backup_task
from otcextensions.sdk.volume_backup.v2 import job as _job


class Proxy(proxy.Proxy):

    skip_discovery = True

    # ======== Backup Policy ========
    def backup_policies(self):
        """Retrieve a generator of backup_policys

        :returns: A generator of backup_policy (:class:`~openstack.
            volume_backup.v2.backup_policy.BackupPolicy`) instances
        """
        return self._list(_backup_policy.BackupPolicy, paginated=False)

    def create_backup_policy(self, **attrs):
        """Create a new backup policy from name and scheduled policy attributes

        :param dict attrs: Keyword arguments which will be used to create a
            :class:`~otcextensions.sdk.volume_backup.v2.backup_policy.BackupPolicy`

        :returns: The results of backup policy creation
        :rtype:
            :class:`~otcextensions.sdk.volume_backup.v2.backup_policy.BackupPolicy`
        """
        return self._create(_backup_policy.BackupPolicy, **attrs)
        # scheduled_policy = _backup_policy.SchedulePolicy.new(**attrs)
        # backup_policy = _backup_policy.BackupPolicy(
        #     name=name, scheduled_policy=scheduled_policy)
        # return backup_policy.create(self, prepend_key=False)

    def update_backup_policy(self, backup_policy, **attrs):
        """update a backup_policy from backup policy attributes

        :param backup_policy: The value can be the ID of a backup_policy or a
            :class:`~otcextensions.sdk.volume_backup.v2.backup_policy.BackupPolicy`
            instance
        :param dict attrs: Keyword arguments which will be used to create a
            :class:`~otcextensions.sdk.volume_backup.v2.backup_policy.ScheduledPolicy`,
            comprised of the properties on the SchedulePolicy class.

        :returns: The results of backup_policy creation
        :rtype:
            :class:`~otcextensions.sdk.volume_backup.v2.backup_policy.BackupPolicy`
        """
        return self._update(_backup_policy.BackupPolicy,
                            backup_policy,
                            prepend_key=False,
                            **attrs)

    def delete_backup_policy(self, backup_policy, ignore_missing=True):
        """Delete a backup policy

        :param backup_policy: The value can be the ID of a backup_policy or a
            :class:`~otcextensions.sdk.volume_backup.v2.backup_policy.BackupPolicy`
            instance
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
            the backup_policy does not exist.
            When set to ``True``, no exception will be set when attempting to
            delete a nonexistent backup_policy.

        :returns: backup_policy been deleted
        :rtype:
            :class:`~otcextensions.sdk.volume_backup.v2.backup_policy.BackupPolicy`
        """
        return self._delete(_backup_policy.BackupPolicy,
                            backup_policy,
                            ignore_missing=ignore_missing)

    def find_backup_policy(self, name_or_id, ignore_missing=True):
        """Find a single backup_policy

        :param name_or_id: The name or ID of a backup_policy
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised
            when the backup_policy does not exist.
            When set to ``True``, no exception will be set when attempting
            to delete a nonexistent backup_policy.

        :returns: ``None``
        """
        if isinstance(name_or_id, _backup_policy.BackupPolicy):
            name_or_id = name_or_id.id
        return self._find(_backup_policy.BackupPolicy, name_or_id,
                          ignore_missing=ignore_missing)

    def execute_policy(self, backup_policy):
        """Execute policy immediately

        ::param backup_policy: The value can be the ID of a backup_policy or a
            :class:`~otcextensions.sdk.volume_backup.v2.backup_policy.BackupPolicy`
            instance
        """
        backup_policy = self._get_resource(_backup_policy.BackupPolicy,
                                           backup_policy)
        return backup_policy.execute(self)

    def enable_policy(self, backup_policy):
        """Enable policy

        ::param backup_policy: The value can be the ID of a backup_policy or a
            :class:`~otcextensions.sdk.volume_backup.v2.backup_policy.BackupPolicy`
            instance
        """
        updated = {
            "scheduled_policy": {
                "status": "ON"
            }
        }
        return self.update_backup_policy(backup_policy, **updated)

    def disable_policy(self, backup_policy):
        """disable policy

        ::param backup_policy: The value can be the ID of a backup_policy or a
            :class:`~otcextensions.sdk.volume_backup.v2.backup_policy.BackupPolicy`
            instance
        """
        updated = {
            "scheduled_policy": {
                "status": "OFF"
            }
        }
        return self.update_backup_policy(backup_policy, **updated)

    def link_resources_to_policy(self, backup_policy, resources):
        """link resource to backup policy

        :param backup_policy: The value can be the ID of a backup_policy or a
            :class:`~otcextensions.sdk.volume_backup.v2.backup_policy.BackupPolicy`
            instance
        :param resources: resources to bound, should be a list of volume id
        :rtype: :class:`~otcextensions.sdk.volume_backup.v2.backup_policy
                    .BackupPolicyResource`
        """
        backup_policy = self._get_resource(_backup_policy.BackupPolicy,
                                           backup_policy)
        policy_resource = _backup_policy.BackupPolicyResource()
        return policy_resource.link(self,
                                    backup_policy.id,
                                    resources)

    def unlink_resources_of_policy(self, backup_policy, resources):
        """Unlink resources of backup policy

        :param backup_policy: The value can be the ID of a backup_policy or a
            :class:`~otcextensions.sdk.volume_backup.v2.backup_policy.BackupPolicy`
            instance
        :param resources: resources to bound, should be a list of volume id
        :rtype: :class:`~otcextensions.sdk.volume_backup.v2.backup_policy.
                        BackupPolicyResource`
        """
        backup_policy = self._get_resource(_backup_policy.BackupPolicy,
                                           backup_policy)
        policy_resource = _backup_policy.BackupPolicyResource()
        return policy_resource.unlink(self,
                                      backup_policy.id,
                                      resources)

    # ======== Misc ========
    def tasks(self, backup_policy, **query):
        """Retrieve a generator of tasks

        :param backup_policy: The value can be the ID of a backup_policy or a
            :class:`~otcextensions.sdk.volume_backup.v2.backup_policy.BackupPolicy`
            instance
        :param dict query: Optional query parameters to be sent to limit the
            resources being returned.
                * ``id``: task id
                * ``job_id``: alternate to id
                * ``status``: includes:``RUNNING``, ``EXECUTE_TIMEOUT``,
                        ``WAITING``, EXECUTE_FAIL``, ``EXECUTE_SUCCESS``
                * ``sort_dir``: ``desc``, ``asc``
                * ``sort_key``: only ``created_at`` support for now
                * ``marker``:  pagination marker
                * ``limit``: pagination limit
                * ``offset``: pagination offset

        :returns: A generator of backup
            (:class:`~otcextensions.sdk.volume_backup.v2.backup.Backup`)
            instances
        """
        backup_policy = self._get_resource(_backup_policy.BackupPolicy,
                                           backup_policy)
        query["policy_id"] = backup_policy.id
        return self._list(_backup_task.BackupTask, paginated=False, **query)

    def get_job(self, job):
        """Get a job detail

        :param job: The value can be the ID of a job
             or a :class:`~otcextensions.sdk.volume_backup.v1.job.Job`
             instance.
        :returns: Backup instance
        :rtype: :class:`~otcextensions.sdk.volume_backup.v1.job.Job`
        """
        return self._get(_job.Job, job)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource


class Sms(resource.Resource):
    base_path = '/notifications/sms'

    # capabilities
    allow_create = True

    #: Message ID, which is unique
    message_id = resource.Body('message_id')
    #: Request ID, which is unique.
    request_id = resource.Body('request_id')
    #: Phone number. The phone number must be preceded by
    #:  a plus sign (+) and a country code.
    endpoint = resource.Body('endpoint')
    #: SMS message content.
    message = resource.Body('message')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from otcextensions.sdk.nat.v2 import _proxy
from otcextensions.sdk.nat.v2 import snat
from otcextensions.sdk.nat.v2 import dnat
from otcextensions.sdk.nat.v2 import gateway

from openstack.tests.unit import test_proxy_base


class TestNatProxy(test_proxy_base.TestProxyBase):
    def setUp(self):
        super(TestNatProxy, self).setUp()
        self.proxy = _proxy.Proxy(self.session)


class TestNatGateway(TestNatProxy):
    def test_gateway_create(self):
        self.verify_create(self.proxy.create_gateway, gateway.Gateway,
                           method_kwargs={'name': 'id'},
                           expected_kwargs={'name': 'id'})

    def test_gateway_delete(self):
        self.verify_delete(self.proxy.delete_gateway,
                           gateway.Gateway, True)

    def test_gateway_get(self):
        self.verify_get(self.proxy.get_gateway, gateway.Gateway)

    def test_gateways(self):
        self.verify_list(self.proxy.gateways, gateway.Gateway)

    def test_gateway_update(self):
        self.verify_update(self.proxy.update_gateway, gateway.Gateway)


class TestNatSnatRule(TestNatProxy):
    def test_snat_rule_create(self):
        self.verify_create(self.proxy.create_snat_rule, snat.Snat,
                           method_kwargs={'name': 'id'},
                           expected_kwargs={'name': 'id'})

    def test_snat_rule_delete(self):
        self.verify_delete(self.proxy.delete_snat_rule,
                           snat.Snat, True)

    def test_snat_rule_get(self):
        self.verify_get(self.proxy.get_snat_rule, snat.Snat)

    def test_snat_rules(self):
        self.verify_list(self.proxy.snat_rules, snat.Snat)


class TestNatDnatRule(TestNatProxy):
    def test_dnat_rule_create(self):
        self.verify_create(self.proxy.create_dnat_rule, dnat.Dnat,
                           method_kwargs={'name': 'id'},
                           expected_kwargs={'name': 'id'})

    def test_dnat_rule_delete(self):
        self.verify_delete(self.proxy.delete_dnat_rule,
                           dnat.Dnat, True)

    def test_dnat_rule_get(self):
        self.verify_get(self.proxy.get_dnat_rule, dnat.Dnat)

    def test_dnat_rules(self):
        self.verify_list(self.proxy.dnat_rules, dnat.Dnat)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
import mock
# from osc_lib import utils as common_utils

from otcextensions.osclient.rds.v1 import configuration
from otcextensions.tests.unit.osclient.rds.v1 import fakes as rds_fakes


class TestRdsListConfigurations(rds_fakes.TestRds):

    column_list_headers = [
        'ID',
        'Name',
        'Description',
        'Datastore Name',
        'Datastore Version Name',
        # 'Str_ID',
        # 'Values'
    ]

    def setUp(self):
        super(TestRdsListConfigurations, self).setUp()

        self.cmd = configuration.ListConfigurations(self.app, None)

        self.app.client_manager.rds.configurations = mock.Mock()

        self.configurations = self.configuration_mock.create_multiple(3)
        self.configuration_data = []
        for s in self.configurations:
            self.configuration_data.append((
                s.id,
                s.name,
                s.description,
                s.datastore_name,
                s.datastore_version_name,
                # s.flavor_detail,
            ))

    def test_list(self):
        arglist = [
        ]

        verifylist = [
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.app.client_manager.rds.configurations.side_effect = [
            self.configurations
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.app.client_manager.rds.configurations.assert_called()

        self.assertEqual(self.column_list_headers, columns)
        self.assertEqual(tuple(self.configuration_data), tuple(data))


class TestRdsShowConfiguration(rds_fakes.TestRds):

    columns = [
        'ID',
        'Name',
        'Description',
        'Datastore Name',
        'Datastore Version Name',
        'Values',
    ]

    def setUp(self):
        super(TestRdsShowConfiguration, self).setUp()

        self.cmd = configuration.ShowConfiguration(self.app, None)

        self.app.client_manager.rds.get_configuration = mock.Mock()
        self.app.client_manager.rds.find_configuration = mock.Mock()

        self.config = self.configuration_mock.create_one()

        self.config_data = (
            self.config.id,
            self.config.name,
            self.config.description,
            self.config.datastore_name,
            self.config.datastore_version_name,
            self.config.values
        )

    def test_show(self):
        arglist = [
            'test_obj',
        ]

        verifylist = [
            ('configuration_group', 'test_obj'),
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.app.client_manager.rds.find_configuration.side_effect = [
            self.config
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.app.client_manager.rds.find_configuration.assert_called()

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.config_data, data)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource


class Member(resource.Resource):
    """CBR Member Resource"""
    resource_key = 'member'
    resources_key = 'members'
    base_path = '/backups/%(backup_id)s/members'

    # capabilities
    allow_create = True
    allow_list = True
    allow_fetch = True
    allow_delete = True
    allow_commit = True

    _query_mapping = resource.QueryParameters(
        'created_at', 'dest_project_id', 'id', 'image_id',
        'status', 'updated_at', 'vault_id')

    #: Properties
    #: Backup ID
    backup_id = resource.URI('backup_id')
    #: Backup sharing time
    #: Example: 2020-02-05T10:38:34.209782
    created_at = resource.Body('created_at')
    #: Destination project ID which the backup is shared
    dest_project_id = resource.Body('dest_project_id')
    #: ID of the image created by using the accepted shared backup_id
    image_id = resource.Body('image_id')
    #: Backup sharing values
    #: values: pending, accepted, rejected
    status = resource.Body('status')
    #: Update time
    #: Example: 2020-02-05T10:38:34.209782
    updated_at = resource.Body('updated_at')
    #: ID of the vault where the shared backup is stored
    vault_id = resource.Body('vault_id')

    #: Updating the resource does not allow the resource key
    def commit(
        self, session, prepend_key=False, has_body=True,
            retry_on_conflict=None, base_path=None, **kwargs):
        return super(Member, self).commit(
            session,
            prepend_key=prepend_key,
            has_body=has_body,
            retry_on_conflict=retry_on_conflict,
            base_path=base_path,
            **kwargs)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from otcextensions.sdk.cts.v1 import _proxy
from otcextensions.sdk.cts.v1 import trace as _trace
from otcextensions.sdk.cts.v1 import tracker as _tracker

from openstack.tests.unit import test_proxy_base


class TestCTSProxy(test_proxy_base.TestProxyBase):

    def setUp(self):
        super(TestCTSProxy, self).setUp()
        self.proxy = _proxy.Proxy(self.session)

    def test_traces(self):
        self.verify_list(
            self.proxy.traces, _trace.Trace,
            expected_kwargs={
                'tracker_name': 'system'
            }
        )

    def test_traces_query(self):
        self.verify_list(
            self.proxy.traces, _trace.Trace,
            method_kwargs={
                'next': '1',
                'limit': '2',
                'service_type': '3',
                'res_type': '4',
                'res_id': '5',
                'res_name': '6',
                'trace_name': '7',
                'from': '8',
                'to': '9',
                'trace_id': '10',
                'level': '11',
                'user': '12'
            },
            expected_kwargs={
                'next': '1',
                'limit': '2',
                'service_type': '3',
                'res_type': '4',
                'res_id': '5',
                'res_name': '6',
                'trace_name': '7',
                'from': '8',
                'to': '9',
                'trace_id': '10',
                'level': '11',
                'user': '12',
                'tracker_name': 'system',
            }
        )

    def test_get_tracker(self):
        self.verify_get(
            self.proxy.get_tracker, _tracker.Tracker,
            expected_args=['resource_id'],
            expected_kwargs={
                'requires_id': False
            },
        )

    def test_create_tracker(self):
        self.verify_create(
            self.proxy.create_tracker, _tracker.Tracker,
        )

    def test_update_tracker(self):
        self.verify_update(
            self.proxy.update_tracker, _tracker.Tracker,
        )

    '''
    def test_delete_tracker(self):
        self.verify_delete(
            self.proxy.delete_tracker, _tracker.Tracker, True,
        )
    '''



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import resource


class TriggerMetadataList(resource.Resource):
    trigger_name = resource.Body('trigger_name', type=str)
    trigger_type = resource.Body('trigger_type', type=str)
    event_type = resource.Body('event_type', type=str)
    event_data = resource.Body('event_data', type=str)


class TempDetail(resource.Resource):
    input = resource.Body('input', type=str)
    output = resource.Body('output', type=str)
    warning = resource.Body('warning', type=str)


class Template(resource.Resource):
    base_path = 'fgs/templates/%(template_id)s'
    # Capabilities
    allow_fetch = True
    # Properties
    template_id = resource.URI('template_id', type=str)

    # Attributes
    #: Template ID.
    id = resource.Body('id', type=str)
    #: Template type.
    type = resource.Body('type', type=int)
    #: Template title.
    title = resource.Body('title', type=str)
    #: Template name.
    template_name = resource.Body('template_name', type=str)
    #: Template description.
    description = resource.Body('description', type=str)
    #: Template runtime.
    runtime = resource.Body('runtime', type=str)
    #: Template handler.
    handler = resource.Body('handler', type=str)
    #: Code type.
    code_type = resource.Body('code_type', type=str)
    #: Code file.
    code = resource.Body('code', type=str)
    #: Maximum duration the function can be executed.
    #: Value range: 3s-259,200s.
    timeout = resource.Body('timeout', type=int)
    #: Memory size.
    memory_size = resource.Body('memory_size', type=int)
    #: Trigger information.
    trigger_metadata_list = resource.Body(
        'trigger_metadata_list', type=list, list_type=TriggerMetadataList)
    #: Template details.
    temp_detail = resource.Body('temp_detail', type=TempDetail)
    #: User data.
    user_data = resource.Body('user_data', type=str)
    #: Encrypted user data.
    encrypted_user_data = resource.Body('encrypted_user_data', type=str)
    #: Dependencies required by the template.
    dependencies = resource.Body('dependencies', type=list)
    #: Template application scenarios.
    scene = resource.Body('scene', type=str)
    #: Cloud service associated with the template.
    service = resource.Body('service', type=str)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from datetime import datetime

from openstack import _log

from otcextensions.sdk.function_graph.v2 import function
from otcextensions.tests.functional.sdk.function_graph import TestFg

_logger = _log.setup_logging('openstack')


class TestFunctionMetric(TestFg):
    def test_list_metrics(self):
        m = list(self.client.metrics(filter='monitor_data'))
        self.assertIsNotNone(len(m[0].duration))

    def test_list_func_metrics(self):
        now = datetime.now().timestamp()
        self.function = self.client.create_function(**TestFg.function_attrs)
        assert isinstance(self.function, function.Function)
        self.addCleanup(
            self.client.delete_function,
            self.function
        )
        m = list(self.client.function_metrics(
            self.function, period=f'{now},{now - 3600}'))
        self.assertGreaterEqual(1, len(m))



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
import uuid

import mock
from keystoneauth1 import adapter

from openstack.tests.unit import base
from otcextensions.sdk.vpcep.v1 import whitelist
from otcextensions.tests.unit.sdk.utils import assert_attributes_equal

VPCEP_SERVICE_ID = uuid.uuid4().hex

EXAMPLE = {
    'id': uuid.uuid4().hex,
    'permission': '*',
    'created_at': '2018-10-18T13:26:40Z',
}


class TestWhitelist(base.TestCase):
    def setUp(self):
        super(TestWhitelist, self).setUp()
        self.sess = mock.Mock(spec=adapter.Adapter)

    def test_basic(self):
        sot = whitelist.Whitelist()
        self.assertEqual('permissions', sot.resources_key)
        self.assertEqual(None, sot.resource_key)
        self.assertEqual(
            '/vpc-endpoint-services/%(endpoint_service_id)s/permissions',
            sot.base_path,
        )
        self.assertTrue(sot.allow_list)
        self.assertFalse(sot.allow_create)
        self.assertFalse(sot.allow_fetch)
        self.assertFalse(sot.allow_commit)
        self.assertFalse(sot.allow_delete)

    def test_make_it(self):
        sot = whitelist.Whitelist(**EXAMPLE)
        assert_attributes_equal(self, sot, EXAMPLE)

    def test_action(self):
        sot = whitelist.Whitelist.existing(
            id=None, endpoint_service_id=VPCEP_SERVICE_ID
        )
        action = 'add'
        domains = ['123', '456']
        formatted_domains = []
        for domain in domains:
            formatted_domains.append('iam:domain::' + domain)
        json_body = {'permissions': formatted_domains, 'action': action}
        response = mock.Mock()
        response.status_code = 200
        response.json.return_value = {'permissions': formatted_domains}
        response.headers = {}
        self.sess.post.return_value = response

        rt = list(sot._action(self.sess, action, domains))
        self.sess.post.assert_called_with(
            'vpc-endpoint-services/%s/permissions/action' % VPCEP_SERVICE_ID,
            json=json_body,
        )
        self.assertEqual(
            rt[0],
            whitelist.Whitelist.existing(permission=formatted_domains[0]),
        )

    def test_add(self):
        sot = whitelist.Whitelist.existing(
            id=None, endpoint_service_id=VPCEP_SERVICE_ID
        )
        sot._action = mock.Mock()

        domains = ['123', '456']
        rt = sot.add(self.sess, domains)
        sot._action.assert_called_with(self.sess, 'add', domains)
        self.assertIsNotNone(rt)

    def test_remove(self):
        sot = whitelist.Whitelist.existing(
            id=None, endpoint_service_id=VPCEP_SERVICE_ID
        )
        sot._action = mock.Mock()

        domains = ['123', '456']
        rt = sot.remove(self.sess, domains)
        sot._action.assert_called_with(self.sess, 'remove', domains)
        self.assertIsNotNone(rt)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import uuid

from openstack import resource

# from otcextensions.sdk.volume_backup.v2 import backup as _backup
from otcextensions.tests.functional.sdk.volume_backup import TestVbs


class TestBackup(TestVbs):
    BACKUP_NAME = "SDK-" + uuid.uuid4().hex
    volume = None
    job = None

    def prepare_volume(self):

        self.SNAPSHOT_NAME = self.getUniqueString()
        self.SNAPSHOT_ID = None
        self.VOLUME_NAME = self.getUniqueString()
        self.VOLUME_ID = None

        volume = self.conn.block_storage.create_volume(
            name=self.VOLUME_NAME,
            size=1)
        resource.wait_for_status(
            session=self.conn.block_storage,
            resource=volume,
            status='available',
            failures=['error'],
            interval=2,
            wait=120)
        # assert isinstance(volume, _volume.Volume)
        self.assertEqual(self.VOLUME_NAME, volume.name)
        self.VOLUME_ID = volume.id
        snapshot = self.conn.block_storage.create_snapshot(
            name=self.SNAPSHOT_NAME,
            volume_id=self.VOLUME_ID)
        resource.wait_for_status(
            session=self.conn.block_storage,
            resource=snapshot,
            status='available',
            failures=['error'],
            interval=2,
            wait=120)
        # assert isinstance(snapshot, _snapshot.Snapshot)
        self.assertEqual(self.SNAPSHOT_NAME, snapshot.name)
        self.SNAPSHOT_ID = snapshot.id

    def cleanup_volume(self):
        snapshot = self.conn.block_storage.get_snapshot(self.SNAPSHOT_ID)
        sot = self.conn.block_storage.delete_snapshot(
            snapshot, ignore_missing=False)
        self.conn.block_storage.wait_for_delete(
            snapshot, interval=2, wait=120)
        self.assertIsNone(sot)
        sot = self.conn.block_storage.delete_volume(
            self.VOLUME_ID, ignore_missing=False)
        self.assertIsNone(sot)

    def test_list_backup(self):
        query = {}
        backups = list(self.client.backups(**query))
        self.assertGreaterEqual(len(backups), 0)

    def test_list_backup_details(self):
        query = {}
        backups = list(self.client.backups(details=True, **query))
        self.assertGreaterEqual(len(backups), 0)

    def test_list_backup_pagination(self):
        query = {}
        backups = list(self.client.backups(**query))
        self.assertGreaterEqual(len(backups), 0)
        query['limit'] = 1
        backups_new = list(self.client.backups(**query))
        self.assertGreaterEqual(len(backups), len(backups_new))

    def test_list_backup_filter_status(self):
        query = {
            'status': 'available'
        }
        backups = list(self.client.backups(details=True, **query))
        if len(backups) > 0:
            for backup in backups:
                self.assertEqual(backup.status, 'available')

    def test_get_backup_from_detail(self):
        query = {}
        backups = list(self.client.backups(details=True, **query))
        if len(backups) > 0:
            backup = self.client.get_backup(backups[0])
            self.assertIsNotNone(backup)

    def test_get_backup(self):
        query = {}
        backups = list(self.client.backups(**query))
        if len(backups) > 0:
            backup = self.client.get_backup(backups[0])
            self.assertIsNotNone(backup)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import copy

from keystoneauth1 import adapter

import mock

from openstack.tests.unit import base

from otcextensions.sdk.volume_backup.v2 import job as _job


EXAMPLE = {
    'status': 'SUCCESS',
    'entities': {
        'bks_create_volume_name': 'autobk_volume',
        'backup_id': 'ba5401a2-7cd2-4c01-8c0d-c936ab412d6d',
        'volume_id': '7e5fdc5a-5e36-4b22-8bcc-7f17037290cc',
        'snapshot_id': 'a77a96bf-dd18-40bf-a446-fdcefc1719ec'
    },
    'job_id': '4010b39b5281d3590152874bfa3b1604',
    'job_type': 'bksCreateBackup',
    'begin_time': '2016-01-28T16:14:09.466Z',
    'end_time': '2016-01-28T16:25:27.690Z',
    'error_code': None,
    'fail_reason': None
}


class TestJob(base.TestCase):

    def setUp(self):
        super(TestJob, self).setUp()
        self.sess = mock.Mock(spec=adapter.Adapter)
        self.sess.get = mock.Mock()

    def test_basic(self):
        sot = _job.Job()
        self.assertEqual(None, sot.resource_key)
        self.assertEqual(None, sot.resources_key)
        self.assertEqual('/jobs',
                         sot.base_path)
        self.assertFalse(sot.allow_list)
        self.assertFalse(sot.allow_create)
        self.assertTrue(sot.allow_get)
        self.assertFalse(sot.allow_update)
        self.assertFalse(sot.allow_delete)

    def test_make_it(self):
        obj = EXAMPLE
        sot = _job.Job.existing(**obj)
        self.assertEqual(obj['job_id'], sot.id)
        self.assertEqual(obj['job_type'], sot.type)

    def test_get(self):
        sot = _job.Job.existing(id=EXAMPLE['job_id'])

        mock_response = mock.Mock()
        mock_response.status_code = 200
        mock_response.headers = {}
        mock_response.json.return_value = copy.deepcopy(EXAMPLE)

        self.sess.get.return_value = mock_response

        result = sot.get(self.sess)

        self.sess.get.assert_called_once_with(
            'jobs/%s' % EXAMPLE['job_id'],
        )

        self.assertDictEqual(
            _job.Job.existing(**EXAMPLE).to_dict(),
            result.to_dict())



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
import mock

from otcextensions.osclient.dcaas.v2 import endpoint_group
from otcextensions.tests.unit.osclient.dcaas.v2 import fakes

from osc_lib import exceptions

from openstackclient.tests.unit import utils as tests_utils


class TestListEndpointGroups(fakes.TestDcaas):

    objects = fakes.FakeEndpointGroup.create_multiple(3)
    column_list_headers = (
        'id',
        'name',
        'description',
        'project id',
        'type',
        'endpoints',
    )
    columns = (
        'id',
        'name',
        'description',
        'project id',
        'type',
        'endpoints'
    )
    data = []
    for s in objects:
        data.append((
            s.id,
            s.name,
            s.description,
            s.project_id,
            s.type,
            s.endpoints,
        ))

    def setUp(self):
        super(TestListEndpointGroups, self).setUp()
        self.cmd = endpoint_group.ListEndpointGroups(self.app, None)
        self.client.endpoint_groups = mock.Mock()
        self.client.api_mock = self.client.endpoint_groups

    def test_list(self):
        arglist = []
        verifylist = []

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.client.api_mock.side_effect = [self.objects]
        columns, data = self.cmd.take_action(parsed_args)

        self.client.api_mock.assert_called_with()
        self.assertEqual(self.column_list_headers, columns)
        self.assertEqual(self.data, list(data))

    def test_list_args(self):
        arglist = [
            '--id', '1',
            '--name', 'test',
            '--description', 'test description',
            '--project_id', 'tid1',
            '--type', 'cidr',
            '--endpoints', '10.2.0.0/24', '10.3.0.0/24',
        ]
        verifylist = [
            ('id', '1'),
            ('name', 'test'),
            ('description', 'test description'),
            ('project_id', 'tid1'),
            ('type', 'cidr'),
            ('endpoints', ["10.2.0.0/24", "10.3.0.0/24"]),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.client.api_mock.side_effect = [self.objects]
        columns, data = self.cmd.take_action(parsed_args)

        self.client.api_mock.assert_called_with(
            id='1',
            name='test',
            description='test description',
            project_id='tid1',
            type='cidr',
            endpoints=["10.2.0.0/24", "10.3.0.0/24"],
        )


class TestCreateEndpointGroup(fakes.TestDcaas):

    _data = fakes.FakeEndpointGroup.create_one()

    columns = (
        'description',
        'endpoints',
        'id',
        'name',
        'project_id',
        'type'
    )

    data = fakes.gen_data(_data, columns)

    def setUp(self):
        super(TestCreateEndpointGroup, self).setUp()
        self.cmd = endpoint_group.CreateEndpointGroup(self.app, None)
        self.client.create_endpoint_group = mock.Mock(return_value=self._data)

    def test_create(self):
        arglist = [
            '--name', 'test',
            '--description', 'test description',
            'tid1',
            'cidr',
            '10.2.0.0/24', '10.3.0.0/24'
        ]
        verifylist = [
            ('name', 'test'),
            ('description', 'test description'),
            ('project_id', 'tid1'),
            ('type', 'cidr'),
            ('endpoints', ["10.2.0.0/24", "10.3.0.0/24"])
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns, data = self.cmd.take_action(parsed_args)
        self.client.create_endpoint_group.assert_called_with(
            project_id='tid1',
            endpoints=["10.2.0.0/24", "10.3.0.0/24"],
            type='cidr',
            name='test',
            description='test description'
        )
        self.assertEqual(self.columns, columns)


class TestShowEndpointGroup(fakes.TestDcaas):

    _data = fakes.FakeEndpointGroup.create_one()

    columns = (
        'description',
        'endpoints',
        'id',
        'name',
        'project_id',
        'type'
    )

    data = fakes.gen_data(_data, columns)

    def setUp(self):
        super(TestShowEndpointGroup, self).setUp()
        self.cmd = endpoint_group.ShowEndpointGroup(self.app, None)
        self.client.find_endpoint_group = mock.Mock(return_value=self._data)

    def test_show_no_options(self):
        arglist = []
        verifylist = []
        self.assertRaises(tests_utils.ParserException, self.check_parser,
                          self.cmd, arglist, verifylist)

    def test_show(self):
        arglist = [
            self._data.id,
        ]
        verifylist = [
            ('endpoint_group', self._data.id)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns, data = self.cmd.take_action(parsed_args)
        self.client.find_endpoint_group.assert_called_with(self._data.id)
        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)

    def test_show_non_existing(self):
        arglist = [
            'unexisted_endpoint_group'
        ]
        verifylist = [
            ('endpoint_group', 'unexisted_endpoint_group')
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        find_mock_result = exceptions.CommandError('Resource Not Found')
        self.client.find_endpoint_group = (
            mock.Mock(side_effect=find_mock_result)
        )
        try:
            self.cmd.take_action(parsed_args)
        except Exception as e:
            self.assertEqual('Resource Not Found', str(e))
        self.client.find_endpoint_group.assert_called_with(
            'unexisted_endpoint_group'
        )


class TestUpdateEndpointGroup(fakes.TestDcaas):

    _data = fakes.FakeEndpointGroup.create_one()

    columns = (
        'description',
        'endpoints',
        'id',
        'name',
        'project_id',
        'type'
    )

    data = fakes.gen_data(_data, columns)

    def setUp(self):
        super(TestUpdateEndpointGroup, self).setUp()
        self.cmd = endpoint_group.UpdateEndpointGroup(self.app, None)
        self.client.find_endpoint_group = mock.Mock(return_value=self._data)
        self.client.update_endpoint_group = mock.Mock(return_value=self._data)
        self.client.api_mock = self.client.update_endpoint_group

    def test_update(self):
        arglist = [
            self._data.id,
            '--name', 'updated_name',
            '--description', 'updated description'
        ]
        verifylist = [
            ('endpoint_group', self._data.id),
            ('name', 'updated_name'),
            ('description', 'updated description')
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns, data = self.cmd.take_action(parsed_args)
        self.client.api_mock.side_effect = [
            self._data
        ]
        self.client.api_mock.assert_called_with(
            self._data.id,
            name='updated_name',
            description='updated description'
        )
        self.assertEqual(self.columns, columns)


class TestDeleteEndpointGroup(fakes.TestDcaas):
    _data = fakes.FakeEndpointGroup.create_one()

    def setUp(self):
        super(TestDeleteEndpointGroup, self).setUp()
        self.client.delete_endpoint_group = mock.Mock(return_value=None)
        self.cmd = endpoint_group.DeleteEndpointGroup(self.app, None)

    def test_delete_by_name(self):
        arglist = [
            self._data.name,
        ]
        verifylist = [
            ('endpoint_group', self._data.name),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.client.find_endpoint_group = (
            mock.Mock(return_value=self._data)
        )
        result = self.cmd.take_action(parsed_args)
        self.client.delete_endpoint_group.assert_called_with(self._data.id)
        self.assertIsNone(result)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from otcextensions.tests.functional.sdk.apig import TestApiG
import json


class TestExportApi(TestApiG):
    gateway = '560de602c9f74969a05ff01d401a53ed'

    def setUp(self):
        super(TestExportApi, self).setUp()

    def test_export_api(self):
        attrs = {
            "env_id": "DEFAULT_ENVIRONMENT_RELEASE_ID",
            "group_id": "ce973ff83ce54ef192c80bde884aa0ac",
            "define": "all"
        }
        self.client.export_api(
            gateway=TestExportApi.gateway,
            full_path='/mnt/c/Users/sand1/api',
            **attrs
        )

    def test_import_api(self):
        with (open('/mnt/c/Users/sand1/PycharmProjects/python-otcextensions/'
                   'otcextensions/tests/functional/sdk/apig/v2/test.json',
                   'r') as f):
            openapi_content = json.load(f)
        attrs = {
            "is_create_group": False,
            "group_id": "ce973ff83ce54ef192c80bde884aa0ac",
            "file_name": "test.json",
            "swagger": openapi_content
        }
        self.client.import_api(
            gateway=TestExportApi.gateway,
            **attrs
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import proxy

from otcextensions.sdk.sdrs.v1 import job as _job
from otcextensions.sdk.sdrs.v1 import active_domains as _active_domains
from otcextensions.sdk.sdrs.v1 import protection_group as _protection_group
from otcextensions.sdk.sdrs.v1 import protected_instance as _p_instance
from otcextensions.sdk.sdrs.v1 import replication_pair as _replication_pair
from otcextensions.sdk.sdrs.v1 import dr_drill as _dr_drill
from otcextensions.sdk.sdrs.v1 import task_center as _task_center
from otcextensions.sdk.sdrs.v1 import quota as _quota


class Proxy(proxy.Proxy):
    skip_discovery = True

    # ======== Job ========
    def get_job(self, job):
        """ Get single SDRS job by UUID.

        :param job: The id or and instance of
            :class:'~otcextensions.sdk.sdrs.v1.job.Job'

        :returns: instance of
            :class: '~class:'otcextensions.sdk.sdrs.v1.job.Job'
        """
        return self._get(_job.Job, job)

    # ======== Active-active domain ========
    def get_domains(self):
        """Retrieve a generator of Active-active domains

        :returns: A generator of active-active domains
            :class: '~otcextensions.sdk.sdrs.v1.active_domains.ActiveDomain'
        """
        return self._list(_active_domains.ActiveDomains)

    # ======== Protection group ========
    def create_protection_group(self, **attrs):
        """Creating a SDRS protection group using attributes

        :param dict attrs:  Keyword arguments which will be used to create
            a :class:`~otcextensions.sdk.sdrs.v1.protection_group.
            ProtectionGroup`, comprised of the properties on
            the ProtectionGroup class.
        :returns: The results of config creation
        :rtype: :class:`~otcextensions.sdk.sdrs.v1.protection_group.
            ProtectionGroup`
        """
        return self._create(_protection_group.ProtectionGroup, **attrs)

    def protection_groups(self, **query):
        """Retrieve a generator of Protection groups

        :param dict query: Optional query parameters to be sent to limit the
            resources being returned.
            * 'availability_zone': Production site AZ
            * `limit`: Number of records displayed per page
            * `marker`: ID of the last record displayed
            * `name`: Protection group name
            * `offset`: Offset value
            * 'query_type': Query type of protection group
            * `status`: Status

        :returns: A generator of protection groups
            :class:
            `~otcextensions.sdk.sdrs.v1.protection_group.ProtectionGroup`
            instances
        """
        return self._list(_protection_group.ProtectionGroup, **query)

    def get_protection_group(self, protection_group):
        """Get the protection group by UUID.

        :param protection_group: key id or an instance of
            :class:
            `~otcextensions.sdk.sdrs.v1.protection_group.ProtectionGroup`

        :returns: instance of
            :class:
            `~otcextensions.sdk.sdrs.v1.protection_group.ProtectionGroup`
        """
        return self._get(
            _protection_group.ProtectionGroup,
            protection_group
        )

    def delete_protection_group(self,
                                protection_group,
                                ignore_missing=True):
        """Delete a single SDRS protection group.

        :param protection_group: The value can be the ID of a protection_group
             or a :class:
             `~otcextensions.sdk.sdrs.v1.protection_group.ProtectionGroup`
             instance.
        :param bool ignore_missing: When set to ``False``
            :class:
            `~otcextensions.sdk.sdrs.v1.protection_group.ProtectionGroup` will
            be raised when the group does not exist.
            When set to ``True``, no exception will be set when attempting to
            delete a nonexistent protection group.
        """
        return self._delete(
            _protection_group.ProtectionGroup,
            protection_group,
            ignore_missing=ignore_missing,
        )

    def find_protection_group(self, name_or_id, ignore_missing=True):
        """Find a single SDRS protection group by name or id

        :param name_or_id: The name or ID of a protection group
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised
            when the group does not exist.
            When set to ``True``, no exception will be set when attempting
            to find a nonexistent protection group.

        :returns: a :class:
            `~otcextensions.sdk.sdrs.v1.protection_group.ProtectionGroup`
        """
        return self._find(
            _protection_group.ProtectionGroup, name_or_id,
            ignore_missing=ignore_missing
        )

    def update_protection_group(self, protection_group, name):
        """Update SDRS protection group name

        :param protection_group: The value can be the ID of a protection group
            or a :class:
            `~otcextensions.sdk.sdrs.v1.protection_group.ProtectionGroup`
            instance.
        :param str name: name to be updated for protection group

        :returns: a :class:
            `~otcextensions.sdk.sdrs.v1.protection_group.ProtectionGroup`
        """
        protection_group = self._get_resource(
            _protection_group.ProtectionGroup, protection_group
        )
        return self._update(
            _protection_group.ProtectionGroup, protection_group, name=name
        )

    def enable_protection(self, protection_group):
        """Enable protection for existing protection group

        :param protection_group: The value can be the ID of a protection group
            or a :class:
            `~otcextensions.sdk.sdrs.v1.protection_group.ProtectionGroup`
            instance.
        :returns: The result is the job id of an action
        """
        protection_group = self._get_resource(
            _protection_group.ProtectionGroup,
            protection_group
        )
        return protection_group.enable_protection_group(
            self,
            protection_group=protection_group.id
        )

    def disable_protection(self, protection_group):
        """Disable protection for existing protection group

        :param protection_group: The value can be the ID of a protection group
            or a :class:
            `~otcextensions.sdk.sdrs.v1.protection_group.ProtectionGroup`
            instance.
        :returns: The result is the job id of an action
        """
        protection_group = self._get_resource(
            _protection_group.ProtectionGroup,
            protection_group
        )
        return protection_group.disable_protection_group(
            self,
            protection_group=protection_group.id
        )

    def perform_failover(self, protection_group):
        """Perform failover for protection group

        :param protection_group: The value can be the ID of a protection group
            or a :class:
            `~otcextensions.sdk.sdrs.v1.protection_group.ProtectionGroup`
            instance.
        :returns: The result is the job id of an action
        """
        protection_group = self._get_resource(
            _protection_group.ProtectionGroup,
            protection_group
        )
        return protection_group.perform_failover(
            self,
            protection_group=protection_group.id
        )

    def perform_planned_failover(self,
                                 protection_group,
                                 priority_station='target'):
        """Perform failover for protection group

        :param protection_group: The value can be the ID of a protection group
            or a :class:
            `~otcextensions.sdk.sdrs.v1.protection_group.ProtectionGroup`
            instance.
        :param priority_station: direction of planned failover
            Values: 'target' or 'source'
        :returns: The result is the job id of an action
        """
        protection_group = self._get_resource(
            _protection_group.ProtectionGroup,
            protection_group
        )
        return protection_group.perform_planned_failover(
            self,
            protection_group=protection_group.id,
            priority_station=priority_station
        )

    # ======== Protected instance ========
    def create_protected_instance(self, **attrs):
        """Creating a protected instance using attributes

        :param dict attrs: Keyword arguments which will be used to create
            a :class:`~otcextensions.sdk.sdrs.v1.
            protected_instance.ProtectedInstance`,
            comprised of the properties on the Protected instance class.
        :returns: The results of config creation
        :rtype: :class `~otcextensions.sdk.sdrs.v1.protected_instance.
            ProtectedInstance`
        """
        return self._create(
            _p_instance.ProtectedInstance,
            **attrs
        )

    def delete_protected_instance(self,
                                  protected_instance,
                                  delete_target_server=False,
                                  delete_target_eip=False,
                                  ignore_missing=True):
        """Delete a single SDRS protected instance.

        :param protected_instance: The value can be the ID of
             a protected instance or a
             :class:
             `~otcextensions.sdk.sdrs.v1.protected_instance.ProtectedInstance`
             instance.
        :param bool delete_target_server: Specifies whether target
            ECS should be deleted after protection group deletion
        :param bool delete_target_eip: Specifies whether target
            EIP should be deleted after protection group deletion
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
            the group does not exist.
            When set to ``True``, no exception will be set when attempting to
            delete a nonexistent protected instance
        :returns: instance of
            :class:
            `~otcextensions.sdk.sdrs.v1.protected_instance.ProtectedInstance`
        """
        res = self._get_resource(_p_instance.ProtectedInstance,
                                 protected_instance)
        return res.delete(self,
                          delete_target_server=delete_target_server,
                          delete_target_eip=delete_target_eip,
                          ignore_missing=ignore_missing)

    def protected_instances(self, **query):
        """Retrieve a generator of Protected instances

        :param dict query: Optional query parameters to be sent to limit the
            resources being returned.
            * 'availability_zone': Production site AZ
            * `limit`: Number of records displayed per page
            * `marker`: ID of the last record displayed
            * `name`: Protection group name
            * `offset`: Offset value
            * 'protected_instance_ids': Protected instance ID list
            * 'query_type': Query type of protected instance
            * 'server_group_id': Protection group ID
            * 'server_group_ids': Protection groups ID list
            * `status`: Status

        :returns: A generator of protected instances
            :class:
            `~otcextensions.sdk.sdrs.v1.protected_instance.ProtectedInstance`
            instances
        """
        return self._list(_p_instance.ProtectedInstance,
                          **query)

    def get_protected_instance(self, instance_id):
        """Get the SDRS protected instance by UUID.

        :param instance_id: key id or an instance of
            :class:
            `~otcextensions.sdk.sdrs.v1.protected_instance.ProtectedInstance`

        :returns: instance of
            :class:
            `~otcextensions.sdk.sdrs.v1.protected_instance.ProtectedInstance`
        """
        return self._get(
            _p_instance.ProtectedInstance,
            instance_id
        )

    def update_protected_instance(self, instance_id, name):
        """Update SDRS protected instance name

        :param instance_id: The value can be the ID of a protected instance
            or a :class:`~otcextensions.sdk.sdrs.v1.protected_instance.
            ProtectedInstance` instance.
        :param str name: name to be updated for protected instance
        :returns: :class:`~otcextensions.sdk.sdrs.v1.protected_instance.
            ProtectedInstance``
        """
        protected_instance = self._get_resource(
            _p_instance.ProtectedInstance, instance_id
        )
        return self._update(
            _p_instance.ProtectedInstance,
            protected_instance,
            name=name
        )

    def find_protected_instance(self, name_or_id, ignore_missing=True):
        """Find a single SDRS protected instance by name or id

        :param name_or_id: The name or ID of a protected instance
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised
            when the instance does not exist.
            When set to ``True``, no exception will be set when attempting
            to find a nonexistent protected instance.
        :returns: Protected instance
        :rtype: :class:`~otcextensions.sdk.sdrs.v1.protected_instance.
            ProtectedInstance`
        """
        return self._find(
            _p_instance.ProtectedInstance,
            name_or_id,
            ignore_missing=ignore_missing
        )

    def attach_replication_pair(self, protected_instance,
                                replication,
                                device='/dev/vdb'):
        """Attach replication pair to protected instance

        :param protected_instance: The value can be the ID of a protected
            instance or :class:
            `~otcextensions.sdk.sdrs.v1.protected_instance.ProtectedInstance`
        :param replication: The value can be the ID of a replication pair
            or :class:
            `~otcextensions.sdk.sdrs.v1.replication_pair.ReplicationPair`
        :param device: Disk device name of replication pair
        """
        protected_instance = self._get_resource(
            _p_instance.ProtectedInstance,
            protected_instance)
        replication = self._get_resource(
            _replication_pair.ReplicationPair, replication)

        return protected_instance.attach_pair(
            self,
            protected_instance=protected_instance.id,
            replication_id=replication.id,
            device=device
        )

    def detach_replication_pair(self,
                                protected_instance,
                                replication):
        """Detach replication pair from protected instance

        :param protected_instance: The value can be the ID of a
            protected instance or a :class:
            `~otcextensions.sdk.sdrs.v1.protected_instance.ProtectedInstance`
        :param replication: The value can be the ID of a replication pair
            or :class:
            `~otcextensions.sdk.sdrs.v1.replication_pair.ReplicationPair`
        """
        protected_instance = self._get_resource(
            _p_instance.ProtectedInstance,
            protected_instance)
        replication = self._get_resource(
            _replication_pair.ReplicationPair, replication)
        return protected_instance.detach_pair(
            self,
            protected_instance=protected_instance.id,
            replication_id=replication.id
        )

    def add_nic(self, protected_instance, subnet_id,
                security_groups=None, ip_address=None):
        """Add NIC to a protected instance

        :param protected_instance: The value can be the ID of a
            protected instance or
            :class:
            `~otcextensions.sdk.sdrs.v1.protected_instance.ProtectedInstance`
        :param str subnet_id: Subnet ID of the NIC to be added
        :param list security_groups: list of security groups to
            be added for NIC in format 'id': 'value'
        :param str ip_address: IP address of NIC
        """
        protected_instance = self._get_resource(
            _p_instance.ProtectedInstance,
            protected_instance)
        return protected_instance.add_nic(
            self,
            protected_instance=protected_instance.id,
            subnet_id=subnet_id,
            security_groups=security_groups,
            ip_address=ip_address
        )

    def delete_nic(self, protected_instance, nic_id):
        """Delete NIC from a protected instance

        :param protected_instance: The value can be the ID of a
            protected instance or a :class:
            `~otcextensions.sdk.sdrs.v1.protected_instance.ProtectedInstance`
        :param str nic_id: ID of a Network interface card to be deleted
        """
        protected_instance = self._get_resource(
            _p_instance.ProtectedInstance,
            protected_instance)
        return protected_instance.delete_nic(
            self,
            protected_instance=protected_instance.id,
            nic_id=nic_id
        )

    def modify_protected_instance(self,
                                  protected_instance,
                                  flavor=None,
                                  production_flavor=None,
                                  dr_flavor=None):
        """Modify server(s) flavor of protected instance

        :param protected_instance: The value can be the ID of
            a protected instance or a :class:
            `~otcextensions.sdk.sdrs.v1.protected_instance.ProtectedInstance`
        :param str flavor: flavor ID for both production and DR sites
        :param str production_flavor: flavor ID for production site
            If 'flavor' is specified this parameter doesn't take effect
        :param str dr_flavor: flavor ID for DR site
            If 'flavor' is specified this parameter doesn't take effect
        """
        protected_instance = self._get_resource(
            _p_instance.ProtectedInstance,
            protected_instance)
        return protected_instance.modify_instance(
            self,
            protected_instance=protected_instance.id,
            flavor=flavor,
            production_flavor=production_flavor,
            dr_flavor=dr_flavor
        )

    # ======== Replication pair ========

    def create_replication_pair(self, **attrs):
        """Create a replication pair using attributes

        :param dict attrs: Keyword arguments which will be used to create
            a :class:`~otcextensions.sdk.sdrs.v1.replication_pair.
            ReplicationPair`, comprised of the properties on the Replication
            Pair class.
        :returns: The results of config creation
        :rtype: :class:`~otcextensions.sdk.sdrs.v1.replication_pair.
            ReplicationPair`
        """
        return self._create(
            _replication_pair.ReplicationPair,
            **attrs
        )

    def delete_replication_pair(self,
                                replication,
                                server_group_id=None,
                                delete_target_volume=False,
                                ignore_missing=True):
        """Delete a single SDRS replication pair

        :param replication: The value can be the ID of a replication pair
             or a :class:
             `~otcextensions.sdk.sdrs.v1.replication_pair.ReplicationPair`
             instance.
        :param bool server_group_id: Protection group ID of replication pair
        :param bool delete_target_volume: Specifies whether DR site
            volume should be deleted after replication pair deletion
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised
            when the group does not exist.
            When set to ``True``, no exception will be set when attempting to
            delete a nonexistent protected instance
        :returns: instance of
            :class:
            `~otcextensions.sdk.sdrs.v1.replication_pair.ReplicationPair`
        """
        res = self._get_resource(_replication_pair.ReplicationPair,
                                 replication)
        return res.delete(self,
                          server_group_id=server_group_id,
                          delete_target_volume=delete_target_volume,
                          ignore_missing=ignore_missing)

    def replication_pairs(self, **query):
        """Retrieve a generator of Replication pairs

        :param dict query: Optional query parameters to be sent to limit the
            resources being returned.
            * 'availability_zone': Production site AZ
            * `limit`: Number of records displayed per page
            * `marker`: ID of the last record displayed
            * `name`: Replication pair name
            * `offset`: Offset value
            * 'protected_instance_id': Protected instance ID
            * 'protected_instance_ids': Protected instances ID list
            * 'query_type': Query type of replication pair
            Values: status_abnormal, general
            * 'server_group_id': Protection group ID
            * 'server_group_ids': Protection groups ID list
            * `status`: Status

        :returns: A generator of replication pairs
            :class:
            `~otcextensions.sdk.sdrs.v1.replication_pair.ReplicationPair`
            instances
        """
        return self._list(_replication_pair.ReplicationPair,
                          **query)

    def get_replication_pair(self, replication):
        """Get the SDRS replication pair by UUID.

        :param replication: key id or an instance of
            :class:
            `~otcextensions.sdk.sdrs.v1.replication_pair.ReplicationPair`

        :returns: instance of
            :class:
            `~otcextensions.sdk.sdrs.v1.replication_pair.ReplicationPair`
        """
        return self._get(_replication_pair.ReplicationPair,
                         replication)

    def find_replication_pair(self, name_or_id, ignore_missing=True):
        """Find a single SDRS replication pair by name or id

        :param name_or_id: The name or ID of a replication pair
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised
            when the pair does not exist.
            When set to ``True``, no exception will be set when attempting
            to find a nonexistent replication pair

        :returns: a :class:
            `~otcextensions.sdk.sdrs.v1.replication_pair.ReplicationPair`
        """
        return self._find(
            _replication_pair.ReplicationPair,
            name_or_id,
            ignore_missing=ignore_missing
        )

    def expand_replication_pair(self, replication, new_size):
        """Expand replication pair

        :param replication: The value can be the ID of a replication pair
            or a :class:
            `~otcextensions.sdk.sdrs.v1.replication_pair.ReplicationPair`
        :param int new_size: Replication pair new size
        """
        replication = self._get_resource(_replication_pair.ReplicationPair,
                                         replication)
        return replication.expand_replication(self,
                                              replication=replication.id,
                                              new_size=new_size)

    def update_replication_pair(self, replication, name):
        """Update SDRS replication pair name

        :param replication: The value can be the ID of a replication pair
            or a :class:`~otcextensions.sdk.sdrs.v1.replication_pair.
            ReplicationPair` instance.
        :param str name: name to be updated for replication pair
        :returns: Replication pair instance
        :rtype: :class:`~otcextensions.sdk.sdrs.v1.replication_pair.
            ReplicationPair`
        """
        replication = self._get_resource(
            _replication_pair.ReplicationPair, replication
        )
        return self._update(
            _replication_pair.ReplicationPair,
            replication,
            name=name
        )

    # ======== Disaster recovery drill ========

    def create_dr_drill(self, **attrs):
        """Creating a disaster recovery drill using attributes

        :param dict attrs: Keyword arguments which will be used to create
            a :class:`~otcextensions.sdk.sdrs.v1.dr_drill.DRDrill`,
            comprised of the properties on the DR drill class.
        :returns: The results of config creation
        :rtype: :class:`~otcextensions.sdk.sdrs.v1.dr_drill.DRDrill`
        """
        return self._create(
            _dr_drill.DRDrill,
            **attrs
        )

    def delete_dr_drill(self, dr_drill, ignore_missing=True):
        """Delete a single SDRS DR drill

        :param dr_drill: The value can be the ID of a dr_drill
             or a :class:`~otcextensions.sdk.sdrs.v1.dr_drill.DRDrill`
             instance.
        :param bool ignore_missing: When set to ``False``
            :class:`~otcextensions.sdk.sdrs.v1.dr_drill.DRDrill` will
            be raised when the dr_drill does not exist.
            When set to ``True``, no exception will be set when attempting
            to delete a nonexistent DR drill
        """
        return self._delete(
            _dr_drill.DRDrill,
            dr_drill,
            ignore_missing=ignore_missing,
        )

    def dr_drills(self, **query):
        """Retrieve a generator of DR drills

        :param dict query: Optional query parameters to be sent to limit the
            resources being returned.
            * 'drill_vpc_id': VPC ID used for DR drill
            * `limit`: Number of records displayed per page
            * `marker`: ID of the last record displayed
            * `name`: DR drill name
            * `offset`: Offset value
            * 'server_group_id': Protection group ID
            * `status`: Status

        :returns: A generator of dr drills
            :class:`~otcextensions.sdk.sdrs.v1.dr_drill.DRDrill`
            instances
        """
        return self._list(_dr_drill.DRDrill,
                          **query)

    def get_dr_drill(self, dr_drill):
        """Get the SDRS DR drill by UUID.

        :param dr_drill: key id or an instance of
            :class:`~otcextensions.sdk.sdrs.v1.dr_drill.DRDrill`

        :returns: instance of
            :class:`~otcextensions.sdk.sdrs.v1.dr_drill.DRDrill`
        """
        return self._get(_dr_drill.DRDrill,
                         dr_drill)

    def find_dr_drill(self, name_or_id, ignore_missing=True):
        """Find a single SDRS DR drill by name or id

        :param name_or_id: The name or ID of a DR drill
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised
            when the pair does not exist.
            When set to ``True``, no exception will be set when attempting
            to find a nonexistent DR drill

        :returns: a :class:`~otcextensions.sdk.sdrs.v1.dr_drill.DRDrill`
        """
        return self._find(
            _dr_drill.DRDrill,
            name_or_id,
            ignore_missing=ignore_missing
        )

    def update_dr_drill(self, dr_drill, name):
        """Update SDRS DR drill name

        :param dr_drill: The value can be the ID of a DR drill
            or a :class:`~otcextensions.sdk.sdrs.v1.dr_drill.DRDrill`
            instance.
        :param str name: name to be updated for DR drill

        :rtype: :class:`~otcextensions.sdk.sdrs.v1.dr_drill.DRDrill`
        """
        dr_drill = self._get_resource(
            _dr_drill.DRDrill, dr_drill
        )
        return self._update(
            _dr_drill.DRDrill,
            dr_drill,
            name=name
        )

    # ======== Failed tasks ========

    def failed_tasks(self, **query):
        """Retrieve a generator of Failed tasks

        :param dict query: Optional query parameters to be sent to limit the
            resources being returned.
            * 'failure_status': query the task failure status
            * `limit`: Number of records displayed per page
            * `marker`: ID of the last record displayed
            * `offset`: Offset value
            * 'resource_name': protection group name
            * `resource_type`: type of the resource
            * `server_group_id`: protection group ID

        :returns: A generator of failed tasks
            :class:`~otcextensions.sdk.sdrs.v1.task_center.FailedTask`
            instances
        """
        return self._list(_task_center.FailedTask,
                          **query)

    def delete_failed_task(self, failed_job_id, ignore_missing=True):
        """Delete a single Failed task

        :param failed_job_id: The value can be the ID of a failed task
             or a :class:`~otcextensions.sdk.sdrs.v1.task_center.FailedTask`
             instance.
        :param bool ignore_missing: When set to ``False``
            :class:`~otcextensions.sdk.sdrs.v1.task_center.FailedTask` will
            be raised when the dr_drill does not exist.
            When set to ``True``, no exception will be set when attempting
            to delete a nonexistent Failed task
        """
        return self._delete(
            _task_center.FailedTask,
            failed_job_id,
            ignore_missing=ignore_missing,
        )

    def delete_all_failed_tasks(self):
        """Delete all failed tasks of all protection
        groups
        """
        endpoint = self.get_endpoint()
        return _task_center.FailedTask.delete_all_tasks(self.session,
                                                        endpoint)

    def delete_protection_group_tasks(self, protection_group):
        """Delete all failed tasks of a single
        protection group

        :param protection_group: The value can be the ID of a protection group
             or a :class:
             `~otcextensions.sdk.sdrs.v1.protection_group.ProtectionGroup`
             instance.
        """
        protection_group = self._get_resource(
            _protection_group.ProtectionGroup,
            protection_group
        )
        endpoint = self.get_endpoint()
        return _task_center.FailedTask.delete_protection_tasks(
            self.session,
            endpoint,
            protection_group.id)

    # ======== Tenant SDRS quota ========

    def quotas(self):
        """Retrieve a generator of SDRS quotas

        :returns: A generator of resource quotas
            :class:`~otcextensions.sdk.sdrs.v1.quota.Quota`
            instances
        """
        return self._list(_quota.Quota)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from otcextensions.sdk.waf.v1 import _proxy
from otcextensions.sdk.waf.v1 import certificate
from otcextensions.sdk.waf.v1 import domain

from openstack.tests.unit import test_proxy_base


class TestWafProxy(test_proxy_base.TestProxyBase):

    def setUp(self):
        super(TestWafProxy, self).setUp()
        self.proxy = _proxy.Proxy(self.session)


class TestWafCertificate(TestWafProxy):
    def test_certificate_create(self):
        self.verify_create(self.proxy.create_certificate,
                           certificate.Certificate,
                           method_kwargs={'name': 'id'},
                           expected_kwargs={'name': 'id',
                                            'prepend_key': False})

    def test_certificate_delete(self):
        self.verify_delete(self.proxy.delete_certificate,
                           certificate.Certificate, True)

    def test_certificate_find(self):
        self.verify_find(self.proxy.find_certificate, certificate.Certificate)

    def test_certificate_get(self):
        self.verify_get(self.proxy.get_certificate, certificate.Certificate)

    def test_certificates(self):
        self.verify_list(self.proxy.certificates, certificate.Certificate)

    def test_certificate_update(self):
        self.verify_update(self.proxy.update_certificate,
                           certificate.Certificate)


class TestWafDomain(TestWafProxy):
    def test_domain_create(self):
        self.verify_create(self.proxy.create_domain,
                           domain.Domain,
                           method_kwargs={'name': 'id'},
                           expected_kwargs={'name': 'id',
                                            'prepend_key': False})

    def test_domain_delete(self):
        self.verify_delete(self.proxy.delete_domain,
                           domain.Domain, True)

    def test_domain_find(self):
        self.verify_find(self.proxy.find_domain, domain.Domain)

    def test_domain_get(self):
        self.verify_get(self.proxy.get_domain, domain.Domain)

    def test_domains(self):
        self.verify_list(self.proxy.domains, domain.Domain)

    def test_domain_update(self):
        self.verify_update(self.proxy.update_domain,
                           domain.Domain)



#   Copyright 2013 Nebula Inc.
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
import logging

from otcextensions import sdk

LOG = logging.getLogger(__name__)

DEFAULT_API_VERSION = '1'

API_VERSION_OPTION = 'os_obs_api_version'
API_NAME = "obs"
API_VERSIONS = {
    "1.0": "openstack.connection.Connection",
    "1": "openstack.connection.Connection",
}


# Required by the OSC plugin interface
def make_client(instance):
    """Returns a client to the ClientManager

    Called to instantiate the requested client version.  instance has
    any available auth info that may be required to prepare the client.

    :param ClientManager instance: The ClientManager that owns the new client
    """

    conn = instance.sdk_connection

    if getattr(conn, 'obs', None) is None:
        LOG.debug('OTC extensions are not registered. Do that now')
        sdk.register_otc_extensions(conn)

    LOG.debug('OBS client initialized using OpenStack OTC SDK: %s',
              conn.obs)
    return conn.obs


# Required by the OSC plugin interface
def build_option_parser(parser):
    """Hook to add global options

    Called from openstackclient.shell.OpenStackShell.__init__()
    after the builtin parser has been initialized.  This is
    where a plugin can add global options such as an API version setting.

    :param argparse.ArgumentParser parser: The parser object that has been
        initialized by OpenStackShell.
    """
    return parser



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import openstack
import uuid
from openstack import resource

from otcextensions.tests.functional import base

_logger = openstack._log.setup_logging('openstack')


class TestDnat(base.BaseFunctionalTest):
    floating_ip = None
    dnat_rule = None
    port = None
    uuid_v4 = uuid.uuid4().hex[:8]
    port_name = uuid_v4 + 'test-dnat-port'
    network_info = None
    gateway = None
    server = None
    keypair = None
    server_name = uuid_v4 + 'test-dnat-server'
    gateway_name = uuid_v4 + 'test-dnat-gateway'
    flavor = "s3.medium.1"
    image = "Standard_Ubuntu_20.04_latest"
    kp_name = uuid_v4 + "test-dnat-kp"
    fixed_ip = "192.168.0.10"
    attrs = {
        "name": gateway_name,
        "spec": "1"
    }
    AZ = 'eu-de-01'

    def _create_network(self):
        cidr = '192.168.0.0/16'
        uuid_v4 = uuid.uuid4().hex[:8]
        vpc_name = 'dnat-test-vpc-' + uuid_v4
        subnet_name = 'dnat-test-subnet-' + uuid_v4

        if not TestDnat.network_info:
            vpc = self.conn.vpc.create_vpc(
                name=vpc_name,
                cidr=cidr
            )
            self.assertEqual(vpc_name, vpc.name)
            vpc_id = vpc.id
            gw, _ = cidr.split("/")
            subnet = self.conn.vpc.create_subnet(
                name=subnet_name,
                vpc_id=vpc.id,
                cidr=vpc.cidr,
                gateway_ip=gw[:-2] + ".1",
                dns_list=[
                    "100.125.4.25",
                    "100.125.129.199",
                ]
            )
            resource.wait_for_status(
                self.conn.vpc, subnet, "ACTIVE", None, 2, 20
            )
            self.assertEqual(subnet_name, subnet.name)
            subnet_id = subnet.id
            network_id = subnet.neutron_network_id

            TestDnat.network_info = {
                'vpc_id': vpc_id,
                'subnet_id': subnet_id,
                'network_id': network_id,
                'subnet': subnet,
                'vpc': vpc
            }
        if not TestDnat.gateway:
            self.attrs['router_id'] = TestDnat.network_info['vpc_id']
            self.attrs['internal_network_id'] = \
                TestDnat.network_info['network_id']
            TestDnat.gateway = self.conn.nat.create_gateway(**self.attrs)
            self.conn.nat.wait_for_gateway(TestDnat.gateway)
            self.assertIsNotNone(TestDnat.gateway)
        if not TestDnat.floating_ip:
            admin_external_net = self.conn.network.find_network(
                name_or_id='admin_external_net')
            self.assertIsNotNone(admin_external_net)
            TestDnat.floating_ip = self.conn.network.create_ip(
                floating_network_id=admin_external_net.id)

        image = self.conn.compute.find_image(self.image)
        flavor = self.conn.compute.find_flavor(self.flavor)

        if not TestDnat.keypair:
            TestDnat.keypair = self.conn.compute.create_keypair(
                name=self.kp_name)

        if not TestDnat.port:
            TestDnat.port = self.conn.network.create_port(
                network_id=TestDnat.network_info['network_id'])

        TestDnat.server = self.conn.compute.create_server(
            name=self.server_name, networks=[{"port": TestDnat.port.id}],
            key_name=self.kp_name, flavorRef=flavor.id, imageRef=image.id)
        self.conn.compute.wait_for_server(TestDnat.server)

    def _destroy_network(self):
        if TestDnat.floating_ip:
            self.conn.network.delete_ip(TestDnat.floating_ip)
            TestDnat.floating_ip = None
        if TestDnat.server:
            # blacklisted now, because OTC had lack of nova apis
            # for releasing fixed_ip
            self.conn.compute.delete_server(TestDnat.server)
            self.conn.compute.wait_for_delete(
                TestDnat.server, interval=5, wait=600)
            TestDnat.server = None
        if TestDnat.keypair:
            self.conn.compute.delete_keypair(TestDnat.keypair)
            self.conn.compute.wait_for_delete(TestDnat.keypair)
            TestDnat.keypair = None
        if TestDnat.gateway:
            self.conn.nat.delete_gateway(gateway=TestDnat.gateway)
            self.conn.nat.wait_for_delete_gateway(TestDnat.gateway)
            TestDnat.gateway = None
        if TestDnat.network_info:
            vpc = TestDnat.network_info['vpc']
            subnet = TestDnat.network_info['subnet']

            resource.wait_for_status(
                self.conn.vpc, subnet, "ACTIVE", None, 2, 20
            )
            self.conn.vpc.delete_subnet(subnet, ignore_missing=False)
            resource.wait_for_delete(self.conn.vpc, subnet, 2, 60)

            sot = self.conn.vpc.delete_vpc(vpc)
            self.assertIsNotNone(sot)

            TestDnat.network_info = None

    def test_01_create_dnat_rule(self):
        self._create_network()
        TestDnat.dnat_rule = self.conn.nat.create_dnat_rule(
            nat_gateway_id=TestDnat.gateway.id,
            floating_ip_id=TestDnat.floating_ip.id,
            network_id=TestDnat.network_info['network_id'],
            private_ip=TestDnat.port.fixed_ips[0]["ip_address"],
            protocol='TCP', internal_service_port=22,
            external_service_port=22)
        self.conn.nat.wait_for_dnat(TestDnat.dnat_rule)
        self.assertIsNotNone(TestDnat.dnat_rule)

    def test_02_list_dnat_rules(self):
        self.dnat_rules = list(self.conn.nat.dnat_rules())
        self.assertGreaterEqual(len(self.dnat_rules), 0)

    def test_03_get_dnat_rule(self):
        dnat_rule = self.conn.nat.get_dnat_rule(TestDnat.dnat_rule.id)
        self.assertEqual(dnat_rule.id, TestDnat.dnat_rule.id)

    def test_04_delete_dnat_rule(self):
        try:
            self.conn.nat.delete_dnat_rule(dnat=TestDnat.dnat_rule)
            self.conn.nat.wait_for_delete_dnat(TestDnat.dnat_rule)
        except openstack.exceptions.InvalidRequest:
            self._destroy_network()
            raise
        self._destroy_network()
        try:
            dnat_rule = self.conn.nat.get_dnat_rule(TestDnat.dnat_rule.id)
        except openstack.exceptions.ResourceNotFound:
            dnat_rule = None
        self.assertIsNone(dnat_rule)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
import argparse

import mock

from otcextensions.osclient.auto_scaling.v1 import config
from otcextensions.tests.unit.osclient.auto_scaling.v1 import fakes


class TestAutoScalingConfig(fakes.TestAutoScaling):

    def setUp(self):
        super(TestAutoScalingConfig, self).setUp()
        self.client = self.app.client_manager.auto_scaling


class TestListAutoScalingConfig(TestAutoScalingConfig):

    configs = fakes.FakeConfig.create_multiple(3)

    columns = ('ID', 'Name')

    data = []

    for s in configs:
        data.append((
            s.id,
            s.name,
        ))

    def setUp(self):
        super(TestListAutoScalingConfig, self).setUp()

        self.cmd = config.ListAutoScalingConfig(self.app, None)

        self.client.configs = mock.Mock()

    def test_list_default(self):
        arglist = [
        ]

        verifylist = [
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.configs.side_effect = [
            self.configs
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.configs.assert_called_once_with()

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, list(data))


class TestShowAutoScalingConfig(TestAutoScalingConfig):

    columns = ['ID', 'Name', 'instance_id', 'instance_name',
               'flavor_id', 'image_id', 'disk',
               'key_name', 'public_ip', 'user_data', 'metadata'
               ]

    _config = fakes.FakeConfig.create_one()

    data = (
        # _config.create_time,
        _config.id,
        _config.name,
        _config.instance_config.instance_id,
        _config.instance_config.instance_name,
        _config.instance_config.flavor_id,
        _config.instance_config.image_id,
        _config.instance_config.disk,
        _config.instance_config.key_name,
        _config.instance_config.public_ip,
        _config.instance_config.user_data,
        _config.instance_config.metadata,
        # format_columns.DictColumn(_config.instance_config).human_readable(),
    )

    def setUp(self):
        super(TestShowAutoScalingConfig, self).setUp()

        self.cmd = config.ShowAutoScalingConfig(self.app, None)

        self.client.find_config = mock.Mock()

    def test_show_default(self):
        arglist = [
            'test_config'
        ]
        verifylist = [
            ('config', 'test_config')
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.find_config.side_effect = [
            self._config
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.find_config.assert_called()

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)


class TestCreateAutoScalingGroup(TestAutoScalingConfig):

    columns = ['ID', 'Name', 'instance_id', 'instance_name',
               'flavor_id', 'image_id', 'disk',
               'key_name', 'public_ip', 'user_data', 'metadata'
               ]

    _config = fakes.FakeConfig.create_one()

    data = (
        _config.id,
        _config.name,
        _config.instance_config.instance_id,
        _config.instance_config.instance_name,
        _config.instance_config.flavor_id,
        _config.instance_config.image_id,
        _config.instance_config.disk,
        _config.instance_config.key_name,
        _config.instance_config.public_ip,
        _config.instance_config.user_data,
        _config.instance_config.metadata,
    )

    def setUp(self):
        super(TestCreateAutoScalingGroup, self).setUp()

        self.cmd = config.CreateAutoScalingConfig(self.app, None)

        self.client.create_config = mock.Mock()

    def test_create_no_details(self):
        arglist = [
            'config_name',
        ]
        verifylist = [
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.assertRaises(argparse.ArgumentTypeError,
                          self.cmd.take_action, parsed_args)

    def test_create_ok(self):
        arglist = [
            'config_name',
            '--flavor', 'some_flavor',
            '--image-id', 'some_image',
            '--disk', 'SYS,SSD,10',
            '--disk', 'DATA,SSD,5',
        ]
        verifylist = [
            ('name', 'config_name'),
            ('flavor', 'some_flavor'),
            ('image_id', 'some_image'),
            ('disk', ['SYS,SSD,10', 'DATA,SSD,5']),
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.client.create_config.side_effect = [
            self._config
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.create_config.assert_called()

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)


class TestDeleteAutoScalingConfig(TestAutoScalingConfig):

    def setUp(self):
        super(TestDeleteAutoScalingConfig, self).setUp()

        self.cmd = config.DeleteAutoScalingConfig(self.app, None)

        self.client.delete_config = mock.Mock()
        self.client.batch_delete_configs = mock.Mock()

    def test_delete_single(self):
        arglist = [
            'config_id'
        ]
        verifylist = [
            ('config', ['config_id'])
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.client.delete_config.side_effect = [{}]

        # Trigger the action
        self.cmd.take_action(parsed_args)

        self.client.delete_config.assert_called_with(
            'config_id',
            ignore_missing=False)

    def test_delete_multiple(self):
        arglist = [
            'config_id1',
            'config_id2'
        ]
        verifylist = [
            ('config', ['config_id1', 'config_id2'])
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.client.batch_delete_configs.side_effect = [{}]

        # Trigger the action
        self.cmd.take_action(parsed_args)

        self.client.delete_config.assert_not_called()
        self.client.batch_delete_configs.assert_called_with(
            ['config_id1', 'config_id2'])



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import sys
import warnings

# -- General configuration ----------------------------------------------------

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom ones.
extensions = [
    'sphinx.ext.autodoc',
    'otcdocstheme',
    'cliff.sphinxext',
]

# Analytics app name
otcdocs_analytics_app = 'otcextensions'

# openstackdocstheme options
#otcdocs_repo_name = 'opentelekomcloud/python-otcextensions'
html_last_updated_fmt = '%Y-%m-%d %H:%M'
html_theme = 'otcdocs'

# TODO(shade) Set this to true once the build-openstack-sphinx-docs job is
# updated to use sphinx-build.
# When True, this will raise an exception that kills sphinx-build.
enforcer_warnings_as_errors = False

# autodoc generation is a bit aggressive and a nuisance when doing heavy
# text edit cycles.
# execute "export SPHINX_DEBUG=1" in your terminal to disable

# The suffix of source filenames.
source_suffix = '.rst'

# The master toctree document.
master_doc = 'index'

# General information about the project.
project = u'python-otcextensions'
copyright = u'2021, Various members of the OpenTelekomCloud'

# A few variables have to be set for the log-a-bug feature.
#   gitsha: The SHA checksum of the bug description. Extracted from git log.
#   bug_tag: Tag for categorizing the bug. Must be set manually.
#   bug_project: Launchpad project to file bugs against.
# These variables are passed to the logabug code via html_context.
git_cmd = "/usr/bin/git log | head -n1 | cut -f2 -d' '"
try:
    gitsha = os.popen(git_cmd).read().strip('\n')
except Exception:
    warnings.warn("Can not get git sha.")
    gitsha = "unknown"

otcdocs_bug_tag = "docs"
pwd = os.getcwd()
# html_context allows us to pass arbitrary values into the html template
html_context = {"pwd": pwd,
                "gitsha": gitsha}

# If true, '()' will be appended to :func: etc. cross-reference text.
add_function_parentheses = True

# If true, the current module name will be prepended to all description
# unit titles (such as .. function::).
add_module_names = True

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = 'native'

autodoc_member_order = "bysource"

# Locations to exclude when looking for source files.
exclude_patterns = []

# -- Options for HTML output ----------------------------------------------

# Don't let openstackdocstheme insert TOCs automatically.
theme_include_auto_toc = False

# Output file base name for HTML help builder.
htmlhelp_basename = '%sdoc' % project

# Grouping the document tree into LaTeX files. List of tuples
# (source start file, target name, title, author, documentclass
# [howto/manual]).
latex_documents = [
    ('index',
     '%s.tex' % project,
     u'%s Documentation' % project,
     u'OpenTelekomCloud', 'manual'),
]

# Include both the class and __init__ docstrings when describing the class
autoclass_content = "both"

# -- Options for cliff.sphinxext plugin ---------------------------------------

autoprogram_cliff_application = 'openstack'

autoprogram_cliff_ignored = [
    '--help', '--format', '--column', '--max-width', '--fit-width',
    '--print-empty', '--prefix', '--noindent', '--quote']



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import resource


class SecurityPolicy(resource.Resource):
    resource_key = 'security_policy'
    resources_key = 'security_policies'
    base_path = '/elb/security-policies'

    # capabilities
    allow_create = True
    allow_fetch = True
    allow_commit = True
    allow_delete = True
    allow_list = True

    _query_mapping = resource.QueryParameters(
        'id', 'description', 'protocols', 'ciphers'
    )

    # Properties
    #: Specifies the security policy id.
    id = resource.Body('id', type=str)
    #: Specifies the security policy name.
    name = resource.Body('name', type=str)
    #: Lists the cipher suites supported by the custom security policy.
    ciphers = resource.Body('ciphers', type=list, list_type=str)
    #: Provides supplementary information about the security policy.
    description = resource.Body('description', type=str)
    #: Specifies the project ID of the custom security policy.
    project_id = resource.Body('project_id', type=str)
    #: Lists the TLS protocols supported by the custom security policy.
    protocols = resource.Body('protocols', type=list, list_type=str)
    #: Specifies the listeners that use the custom security policies.
    listeners = resource.Body('listeners', type=list, list_type=dict)
    #: Specifies the time when the custom security policy was created.
    created_at = resource.Body('created_at', type=str)
    #: Specifies the time when the custom security policy was updated..
    updated_at = resource.Body('updated_at', type=str)
